# python3.7

"""Plugin module for pytest.

This enables easier unit tests for applications that use both Curio and Pytest. If you have Curio
installed, you have the plugin and can write unit tests per the example below.

Provides a fixture named `kernel`, and a marker (pytest.mark.curio) that will run a bare coroutine
in a new Kernel instance.

Example:

    from curio import sleep
    import pytest

    # Use marker

    @pytest.mark.curio
    async def test_coro():
        await sleep(1)


    # Use kernel fixture

    def test_app(kernel):

        async def my_aapp():
            await sleep(1)

        kernel.run(my_aapp)
"""

import inspect
import functools

import pytest

from curio import Kernel
from curio import meta
from curio import monitor
from curio.debug import longblock, logcrash


def _is_coroutine(obj):
    """Check to see if an object is really a coroutine."""
    return meta.iscoroutinefunction(obj) or inspect.isgeneratorfunction(obj)


def pytest_configure(config):
    """Inject documentation."""
    config.addinivalue_line("markers",
                            "curio: "
                            "mark the test as a coroutine, it will be run using a Curio kernel.")


@pytest.mark.tryfirst
def pytest_pycollect_makeitem(collector, name, obj):
    """A pytest hook to collect coroutines in a test module."""
    if collector.funcnamefilter(name) and _is_coroutine(obj):
        item = pytest.Function.from_parent(collector, name=name)
        if 'curio' in item.keywords:
            return list(collector._genfunctions(name, obj))


@pytest.hookimpl(tryfirst=True, hookwrapper=True)
def pytest_pyfunc_call(pyfuncitem):
    """Run curio marked test functions in a Curio kernel instead of a normal function call.
    """
    if pyfuncitem.get_closest_marker('curio'):
        pyfuncitem.obj = wrap_in_sync(pyfuncitem.obj)
    yield


def wrap_in_sync(func):
    """Return a sync wrapper around an async function executing it in a Kernel."""
    @functools.wraps(func)
    def inner(**kwargs):
        coro = func(**kwargs)
        Kernel().run(coro, shutdown=True)
    return inner


# Fixture for explicitly running in Kernel instance.
@pytest.fixture(scope='session')
def kernel(request):
    """Provide a Curio Kernel object for running co-routines."""
    k = Kernel(debug=[longblock, logcrash])
    m = monitor.Monitor(k)
    request.addfinalizer(lambda: k.run(shutdown=True))
    request.addfinalizer(m.close)
    return k



from curio import run, spawn, TaskGroup, Queue, tcp_server

import logging
log = logging.getLogger(__name__)

messages = Queue()
subscribers = set()

async def dispatcher():
    while True:
        msg = await messages.get()
        for q in subscribers:
            await q.put(msg)

async def publish(msg, local):
    log.info('%r published %r', local['address'], msg)
    await messages.put(msg)

async def outgoing(client_stream):
    queue = Queue()
    try:
        subscribers.add(queue)
        while True:
            name, msg = await queue.get()
            await client_stream.write(name + b':' + msg)
    finally:
        subscribers.discard(queue)

async def incoming(client_stream, name, local):
    async for line in client_stream:
        await publish((name, line), local)

async def chat_handler(client, addr):
    log.info('Connection from %r', addr)
    local = { 'address': addr }
    async with client:
        client_stream = client.as_stream()
        await client_stream.write(b'Your name: ')
        name = (await client_stream.readline()).strip()
        await publish((name, b'joined\n'), local)

        async with TaskGroup(wait=any) as workers:
            await workers.spawn(outgoing, client_stream)
            await workers.spawn(incoming, client_stream, name, local)

        await publish((name, b'has gone away\n'), local)

    log.info('%r connection closed', addr)

async def chat_server(host, port):
    async with TaskGroup() as g:
        await g.spawn(dispatcher)
        await g.spawn(tcp_server, host, port, chat_handler)

if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    run(chat_server('', 25000))



# test_kernel.py

import time
import pytest
from curio import *
kernel_clock = clock
from curio import traps

def test_hello(kernel):

    async def hello():
        return 'hello'

    result = kernel.run(hello)
    assert result == 'hello'

def test_raise(kernel):
    class Error(Exception):
        pass

    async def boom():
        raise Error()

    try:
        kernel.run(boom)
        assert False, 'boom() did not raise'
    except Error as e:
        pass
    except:
        assert False, 'boom() raised wrong error'

def test_sleep(kernel):
    start = end = 0
    async def main():
        nonlocal start, end
        start = time.time()
        await sleep(0.5)
        end = time.time()

    kernel.run(main)
    elapsed = end - start
    assert elapsed > 0.5

def test_clock(kernel):
    async def main():
        start = await clock()
        await sleep(0.1)
        end = await clock()
        assert (end - start) >= 0.1
    kernel.run(main)

def test_sleep_cancel(kernel):
    cancelled = False

    async def sleeper():
        nonlocal cancelled
        try:
            await sleep(1)
            assert False
        except CancelledError:
            cancelled = True

    async def main():
        task = await spawn(sleeper)
        await sleep(0.1)
        await task.cancel()

    kernel.run(main)
    assert cancelled

def test_sleep_timeout(kernel):
    cancelled = True

    async def sleeper():
        nonlocal cancelled
        try:
            await timeout_after(0.1, sleep, 1)
            assert False
        except TaskTimeout:
            cancelled = True

    async def main():
        task = await spawn(sleeper)
        await task.join()

    kernel.run(main)
    assert cancelled

def test_sleep_ignore_timeout(kernel):
    async def sleeper():
        cancelled = False

        if await ignore_after(0.1, sleep(1)) is None:
            cancelled = True
        assert cancelled

        cancelled = False
        async with ignore_after(0.1) as s:
            await sleep(1)
        if s.result is None:
            cancelled = True

        assert cancelled

    async def main():
        task = await spawn(sleeper)
        await task.join()

    kernel.run(main)

def test_sleep_notimeout(kernel):
    async def sleeper():
        try:
            await timeout_after(0.5, sleep(0.1))
            assert True
        except TaskTimeout:
            assert False
        await sleep(0.5)
        assert True

    async def main():
        task = await spawn(sleeper)
        await task.join()

    kernel.run(main)

def test_task_join(kernel):
    async def child():
        return 37

    async def main():
        task = await spawn(child)
        r = await task.join()
        assert r == 37

    kernel.run(main)

def test_task_join_error(kernel):
    async def child():
        int('bad')

    async def main():
        task = await spawn(child)
        try:
            r = await task.join()
            assert False
        except TaskError as e:
            assert isinstance(e.__cause__, ValueError)

    kernel.run(main)

def test_task_cancel(kernel):
    cancelled = False
    async def child():
        nonlocal cancelled
        try:
            await sleep(0.5)
            assert False
        except CancelledError:
            cancelled = True

    async def main():
        task = await spawn(child)
        await task.cancel()
        assert cancelled

    kernel.run(main)

def test_task_cancel_poll(kernel):
    results = []

    async def child():
        async with disable_cancellation():
            await sleep(0.1)
            results.append('success')
            if await check_cancellation():
                results.append('cancelled')
            else:
                assert False

    async def main():
        task = await spawn(child)
        await task.cancel()
        results.append('done')

    kernel.run(main)
    assert results == ['success', 'cancelled', 'done']

def test_task_cancel_not_blocking(kernel):
    async def child(e1, e2):
        await e1.set()
        try:
            await sleep(1000)
        except CancelledError:
            await e2.wait()
            raise

    async def main():
        e1 = Event()
        e2 = Event()
        task = await spawn(child, e1, e2)
        await e1.wait()
        await task.cancel(blocking=False)
        await e2.set()
        try:
            await task.join()
        except TaskError as e:
            assert isinstance(e.__cause__, CancelledError)

    kernel.run(main)


def test_task_cancel_join(kernel):
    child_evt = Event()
    async def child():
        await child_evt.wait()
        assert False

    async def main():
        task = await spawn(child)
        await sleep(0)
        await task.cancel()

        # Try joining with a cancelled task. Should raise a TaskError
        try:
            await task.join()
        except TaskError as e:
            assert isinstance(e.__cause__, CancelledError)
        else:
            assert False
        assert True

    kernel.run(main)

def test_task_cancel_join_wait(kernel):
    evt = Event()

    async def child():
        await evt.wait()

    async def canceller(task):
        await task.cancel()

    async def main():
        task1 = await spawn(child)
        task2 = await spawn(canceller, task1)
        await task2.join()
        assert not evt.is_set()
        try:
            await task1.join()     # Should raise TaskError... with CancelledError as cause
            assert False
        except TaskError as e:
            assert isinstance(e.__cause__, CancelledError)
        else:
            assert False

    kernel.run(main)

def test_task_child_cancel(kernel):
    results = []

    async def child():
        results.append('start')
        try:
            await sleep(0.5)
            results.append('end')
        except CancelledError:
            results.append('child cancelled')

    async def parent():
        try:
            child_task = await spawn(child)
            await sleep(0.5)
            results.append('end parent')
        except CancelledError:
            await child_task.cancel()
            results.append('parent cancelled')

    async def grandparent():
        try:
            parent_task = await spawn(parent)
            await sleep(0.5)
            results.append('end grandparent')
        except CancelledError:
            await parent_task.cancel()
            results.append('grandparent cancelled')

    async def main():
        task = await spawn(grandparent)
        await sleep(0.1)
        results.append('cancel start')
        await sleep(0.1)
        results.append('cancelling')
        await task.cancel()
        results.append('done')

    kernel.run(main)

    assert results == [
        'start',
        'cancel start',
        'cancelling',
        'child cancelled',
        'parent cancelled',
        'grandparent cancelled',
        'done',
    ]


def test_task_ready_cancel(kernel):
    # This tests a tricky corner case of a task cancelling another task that's also
    # on the ready queue.
    results = []

    async def child():
        try:
            results.append('child sleep')
            await sleep(1.0)
            results.append('child slept')
            await sleep(1.0)
            results.append('should not see this')
        except CancelledError:
            results.append('child cancelled')

    async def parent():
        task = await spawn(child)
        results.append('parent sleep')
        await sleep(0.5)
        results.append('cancel start')
        await task.cancel()
        results.append('cancel done')

    async def main():
        task = await spawn(parent)
        await sleep(0.1)
        time.sleep(1)      # Forced block of the event loop. Both tasks should awake when we come back
        await sleep(0.1)

    kernel.run(main)

    assert results == [
        'parent sleep',
        'child sleep',
        'cancel start',
        'child slept',
        'child cancelled',
        'cancel done'
    ]


def test_double_cancel(kernel):
    results = []

    async def sleeper():
        results.append('start')
        try:
            await sleep(1)
            results.append('not here')
        except CancelledError:
            results.append('cancel request')
            await sleep(1)
            results.append('cancelled')

    async def main():
        task = await spawn(sleeper)
        await sleep(0.5)
        try:
            await timeout_after(1, task.cancel())
        except TaskTimeout:
            results.append('retry')
            await task.cancel()    # This second cancel should not abort any operation in sleeper
            results.append('done cancel')

    kernel.run(main)
    assert results == [
        'start',
        'cancel request',
        'retry',
        'cancelled',
        'done cancel'
    ]


def test_nested_timeout(kernel):
    results = []

    async def coro1():
        results.append('coro1 start')
        await sleep(1)
        results.append('coro1 done')

    async def coro2():
        results.append('coro2 start')
        await sleep(1)
        results.append('coro2 done')

    # Parent should cause a timeout before the child.
    # Results in a TimeoutCancellationError instead of a normal TaskTimeout
    async def child():
        try:
            await timeout_after(5, coro1())
            results.append('coro1 success')
        except TaskTimeout:
            results.append('coro1 timeout')
        except TimeoutCancellationError:
            results.append('coro1 timeout cancel')

        await coro2()
        results.append('coro2 success')

    async def parent():
        try:
            await timeout_after(0.75, child())
        except TaskTimeout:
            results.append('parent timeout')

    kernel.run(parent)
    assert results == [
        'coro1 start',
        'coro1 timeout cancel',
        'coro2 start',
        'parent timeout'
    ]


def test_nested_context_timeout(kernel):
    results = []

    async def coro1():
        results.append('coro1 start')
        await sleep(1)
        results.append('coro1 done')

    async def coro2():
        results.append('coro2 start')
        await sleep(1)
        results.append('coro2 done')

    # Parent should cause a timeout before the child.
    # Results in a TimeoutCancellationError instead of a normal TaskTimeout
    async def child():
        try:
            async with timeout_after(5):
                await coro1()
            results.append('coro1 success')
        except TaskTimeout:
            results.append('coro1 timeout')
        except TimeoutCancellationError:
            results.append('coro1 timeout cancel')

        await coro2()
        results.append('coro2 success')

    async def parent():
        try:
            async with timeout_after(0.75):
                await child()
        except TaskTimeout:
            results.append('parent timeout')

    kernel.run(parent)
    assert results == [
        'coro1 start',
        'coro1 timeout cancel',
        'coro2 start',
        'parent timeout'
    ]

def test_nested_context_timeout2(kernel):
    async def coro1():
        try:
            async with timeout_after(10):
                await sleep(5)
        except CancelledError as e:
            assert isinstance(e, TimeoutCancellationError)
            raise
        else:
            assert False

    async def coro2():
        try:
            async with timeout_after(15):
                await coro1()
        except CancelledError as e:
            assert isinstance(e, TimeoutCancellationError)
            raise
        else:
            assert False

    async def parent():
        try:
            async with timeout_after(0.1):
                await coro2()
        except Exception as e:
            assert isinstance(e, TaskTimeout)
        else:
            assert False

    kernel.run(parent)

def test_nested_context_timeout3(kernel):
    async def coro1():
        try:
            await timeout_after(10, sleep, 5)
        except CancelledError as e:
            assert isinstance(e, TimeoutCancellationError)
            raise
        else:
            assert False

    async def coro2():
        try:
            await timeout_after(15, coro1)
        except CancelledError as e:
            assert isinstance(e, TimeoutCancellationError)
            raise
        else:
            assert False

    async def parent():
        try:
            await timeout_after(0.1, coro2)
        except Exception as e:
            assert isinstance(e, TaskTimeout)
        else:
            assert False

    kernel.run(parent)

def test_nested_timeout_uncaught(kernel):
    results = []

    async def coro1():
        results.append('coro1 start')
        await sleep(5)
        results.append('coro1 done')

    async def child():
        # This will cause a TaskTimeout, but it's uncaught
        await timeout_after(1, coro1())

    async def parent():
        try:
            await timeout_after(10, child())
        except TaskTimeout:
            results.append('parent timeout')
        except UncaughtTimeoutError:
            results.append('uncaught timeout')

    kernel.run(parent)
    assert results == [
        'coro1 start',
        'uncaught timeout'
    ]


def test_nested_context_timeout_uncaught(kernel):
    results = []

    async def coro1():
        results.append('coro1 start')
        await sleep(5)
        results.append('coro1 done')

    async def child():
        # This will cause a TaskTimeout, but it's uncaught
        async with timeout_after(1):
            await coro1()

    async def parent():
        try:
            async with timeout_after(10):
                await child()
        except TaskTimeout:
            results.append('parent timeout')
        except UncaughtTimeoutError:
            results.append('uncaught timeout')

    kernel.run(parent)
    assert results == [
        'coro1 start',
        'uncaught timeout'
    ]


def test_nested_timeout_none(kernel):
    results = []

    async def coro1():
        results.append('coro1 start')
        await sleep(2)
        results.append('coro1 done')

    async def coro2():
        results.append('coro2 start')
        await sleep(1)
        results.append('coro2 done')

    async def child():
        await timeout_after(None, coro1())
        results.append('coro1 success')
        await coro2()
        results.append('coro2 success')

    async def parent():
        try:
            await timeout_after(1, child())
        except TaskTimeout:
            results.append('parent timeout')

    kernel.run(parent)
    assert results == [
        'coro1 start',
#        'coro1 done',
#        'coro1 success',
#        'coro2 start',
        'parent timeout'
    ]


def test_task_run_error(kernel):
    results = []

    async def main():
        int('bad')

    try:
        kernel.run(main)
        assert False, "Exception not raised"
    except ValueError as e:
        pass
    except:
        assert False, "Wrong exception raised"

def test_sleep_0_starvation(kernel):
    # This task should not block other tasks from running, and should be
    # cancellable. We used to have a bug where neither were true...
    async def loop_forever():
        while True:
            print("Sleeping 0")
            await sleep(0)

    async def io1(sock):
        await sock.recv(1)
        await sock.send(b"x")
        await sock.recv(1)

    async def io2(sock):
        await sock.send(b"x")
        await sock.recv(1)
        await sock.send(b"x")

    async def main():
        loop_task = await spawn(loop_forever)
        await sleep(0)
        import curio.socket
        sock1, sock2 = curio.socket.socketpair()
        io1_task = await spawn(io1, sock1)
        io2_task = await spawn(io2, sock2)
        await io1_task.join()
        await io2_task.join()
        await loop_task.cancel()

    kernel.run(main)


def test_ping_pong_starvation(kernel):
    # It used to be that two of these tasks could starve out other tasks
    async def pingpong(inq, outq):
        while True:
            await outq.put(await inq.get())

    async def i_will_survive():
        for _ in range(10):
            await sleep(0)
        return "i survived!"

    async def main():
        q1 = Queue()
        q2 = Queue()
        await q1.put("something")
        pp1 = await spawn(pingpong, q1, q2)
        pp2 = await spawn(pingpong, q2, q1)
        iws = await spawn(i_will_survive)

        assert (await iws.join()) == "i survived!"
        await pp1.cancel()
        await pp2.cancel()

    kernel.run(main)

def test_task_cancel_timeout(kernel):
    # Test that cancellation also cancels timeouts
    results = []

    async def coro():
        try:
            await sleep(5)
        except CancelledError:
            results.append('cancelled')
            await sleep(1)
            results.append('done cancel')
            raise

    async def child():
        results.append('child')
        try:
            async with timeout_after(1):
                 await coro()
        except TaskTimeout:
            results.append('timeout')

    async def main():
        task = await spawn(child)
        await sleep(0.5)
        await task.cancel()

    kernel.run(main)
    assert results == [ 'child', 'cancelled', 'done cancel' ]

def test_reentrant_kernel(kernel):
    async def child():
        pass

    async def main():
        with pytest.raises(RuntimeError):
            kernel.run(child)

    kernel.run(main)

from curio.traps import *

def test_pending_cancellation(kernel):
    async def main():
        self = await _get_current()
        self.cancel_pending = CancelledError()

        with pytest.raises(CancelledError):
            await _read_wait(None)

        self.cancel_pending = CancelledError()
        with pytest.raises(CancelledError):
            await _future_wait(None)

        self.cancel_pending = CancelledError()
        with pytest.raises(CancelledError):
            await _scheduler_wait(None, None)

        self.cancel_pending = TaskTimeout()
        try:
            await _unset_timeout(None)
            assert True
        except TaskTimeout:
            assert False

    kernel.run(main)

from functools import partial

def test_single_stepping(kernel):
    value = 0
    async def child():
        nonlocal value
        await sleep(0)
        value = 1
        await sleep(0.1)
        value = 2

    task = kernel.run(partial(spawn, child, daemon=True))
    while value < 1:
        kernel.run()
    assert True
    time.sleep(0.2)
    kernel.run()
    assert value == 2

def test_io_registration(kernel):
    # Tests some tricky corner cases of the kernel that are difficult
    # to get to under normal socket usage
    import socket
    s1, s2 = socket.socketpair()
    s1.setblocking(False)
    s2.setblocking(False)

    # Fill the send buffer
    while True:
        try:
            s1.send(b'x'*100000)
        except BlockingIOError:
            break

    async def reader1():
        await traps._read_wait(s1.fileno())
        data = s1.recv(100)
        assert data == b'hello'

    async def writer1():
        await traps._write_wait(s1.fileno())
        assert False

    async def writer2():
        with pytest.raises(WriteResourceBusy):
            await traps._write_wait(s1.fileno())

    async def main():
        t0 = await spawn(reader1)
        t1 = await spawn(writer1)
        t2 = await spawn(writer2)
        await t2.join()
        await t1.cancel()
        s2.send(b'hello')
        await t0.join()
        s1.close()
        s2.close()

    kernel.run(main)

from functools import partial

def test_coro_partial(kernel):
    async def func(x, y, z):
        assert x == 1
        assert y == 2
        assert z == 3
        return True

    async def main():
        assert await func(1, 2, 3)
        assert await ignore_after(1, func(1,2,3))
        assert await ignore_after(1, func, 1, 2, 3)
        assert await ignore_after(1, partial(func, 1, 2), 3)
        assert await ignore_after(1, partial(func, z=3), 1, 2)

        # Try spawns
        t = await spawn(func(1,2,3))
        assert await t.join()

        t = await spawn(func, 1, 2, 3)
        assert await t.join()

        t = await spawn(partial(func, 1, 2), 3)
        assert await t.join()

        t = await spawn(partial(func, z=3), 1, 2)
        assert await t.join()

    kernel.run(main)

def test_custom_cancel(kernel):
    class CustomCancelled(CancelledError):
        pass

    evt = Event()
    async def child():
        try:
            await evt.wait()
        except CustomCancelled:
            assert True
        except:
            assert False
        else:
            assert False

    async def main():
        t = await spawn(child)
        await t.cancel(exc=CustomCancelled)

    kernel.run(main)

def test_timeout_badness(kernel):
    import time
    async def main():
        async with timeout_after(0.1):
            time.sleep(0.2)   # Timeout will take too long. Should issue a warning.

        # Execution should make it here.  There were no blocking operations to cancel.
        # It makes no sense to issue a cancellation on the next operation because we're
        # Likely out of the timeout block
        assert True

    kernel.run(main)

def test_kernel_no_shutdown():
    # Code coverage test
    k = Kernel()
    del k

    with Kernel() as k:
        pass

    with pytest.raises(RuntimeError):
        k.run()


def test_kernel_exit():
    # Code coverage test
    async def main():
        raise SystemExit()

    with pytest.raises(SystemExit):
         with Kernel() as k:
             k.run(main)


def test_kernel_badtrap():
    # Code coverage test
    async def main():
        from curio.traps import _kernel_trap
        await _kernel_trap('bogus', 1)

    with pytest.raises(KeyError):
         with Kernel() as k:
             k.run(main)

def test_kernel_multischedule(kernel):
    async def sleeper():
        try:
            async with timeout_after(0.5):
                await sleep(0.25)
        except TaskTimeout:
            assert False

        await sleep(0.1)    # Should not crash!
        return True

    async def main():
        import time
        t = await spawn(sleeper)
        await sleep(0.1)
        time.sleep(1)     # Force time clock to elapse past both the sleep and outer timeout
        r = await t.join()
        assert r

    kernel.run(main)

def test_kernel_debug():
    from curio.debug import schedtrace, traptrace
    async def hello():
        await sleep(0)

    with Kernel(debug=[schedtrace,traptrace]) as k:
         k.run(hello)

    with Kernel(debug=True) as k:
         k.run(hello)

    with Kernel(debug=schedtrace) as k:
         k.run(hello)

    with Kernel(debug=schedtrace(filter='none')) as k:
        k.run(hello)








# test_socket.py

import os

from curio import *
from curio.socket import *
from curio import io
import pytest
import sys

def test_tcp_echo(kernel, portno):
    results = []
    async def server(address):
        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        sock.listen(5)
        results.append('accept wait')
        client, addr = await sock.accept()
        results.append('accept done')
        await spawn(handler, client)
        await sock.close()

    async def handler(client):
        results.append('handler start')
        while True:
            results.append('recv wait')
            data = await client.recv(100)
            if not data:
                break
            results.append(('handler', data))
            await client.sendall(data)
        results.append('handler done')
        await client.close()

    async def client(address):
        results.append('client start')
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.send(b'Msg1')
        await sleep(0.1)
        resp = await sock.recv(100)
        results.append(('client', resp))
        await sock.send(b'Msg2')
        await sleep(0.1)
        resp = await sock.recv(100)
        results.append(('client', resp))
        results.append('client close')
        await sock.close()

    async def main():
        async with TaskGroup() as g:
            await g.spawn(server, ('', portno))
            await g.spawn(client, ('localhost', portno))

    kernel.run(main())

    assert results == [
        'accept wait',
        'client start',
        'accept done',
        'handler start',
        'recv wait',
        ('handler', b'Msg1'),
        'recv wait',
        ('client', b'Msg1'),
        ('handler', b'Msg2'),
        'recv wait',
        ('client', b'Msg2'),
        'client close',
        'handler done'
    ]


def test_tcp_file_echo(kernel, portno):
    results = []
    async def server(address):
        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        sock.listen(5)
        results.append('accept wait')
        client, addr = await sock.accept()
        results.append('accept done')
        await spawn(handler, client)
        await sock.close()

    async def handler(client):
        results.append('handler start')
        stream = client.as_stream()
        async for line in stream:
            results.append(('handler', line))
            await stream.write(line)
        results.append('handler done')
        await client.close()

    async def client(address):
        results.append('client start')
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        stream = sock.as_stream()
        await stream.write(b'Msg1\n')
        await sleep(0.1)
        resp = await stream.read(100)
        results.append(('client', resp))
        await stream.write(b'Msg2\n')
        await sleep(0.1)
        resp = await stream.read(100)
        results.append(('client', resp))
        results.append('client close')
        await sock.close()

    async def main():
        async with TaskGroup() as g:
             await g.spawn(server, ('', portno))
             await g.spawn(client, ('localhost', portno))

    kernel.run(main())

    assert results == [
        'accept wait',
        'client start',
        'accept done',
        'handler start',
        ('handler', b'Msg1\n'),
        ('client', b'Msg1\n'),
        ('handler', b'Msg2\n'),
        ('client', b'Msg2\n'),
        'client close',
        'handler done'
    ]


def test_udp_echo(kernel):
    results = []
    async def server(address):
        sock = socket(AF_INET, SOCK_DGRAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        results.append('recvfrom wait')
        data, addr = await sock.recvfrom(8192)
        results.append(('server', data))
        await sock.sendto(data, addr)
        await sock.close()
        results.append('server close')

    async def client(address):
        results.append('client start')
        sock = socket(AF_INET, SOCK_DGRAM)
        results.append('client send')
        await sock.sendto(b'Msg1', address)
        data, addr = await sock.recvfrom(8192)
        results.append(('client', data))
        await sock.close()
        results.append('client close')

    async def main():
        async with TaskGroup() as g:
            await g.spawn(server, ('', 25000))
            await g.spawn(client, ('localhost', 25000))

    kernel.run(main())

    assert results == [
        'recvfrom wait',
        'client start',
        'client send',
        ('server', b'Msg1'),
        'server close',
        ('client', b'Msg1'),
        'client close'
    ]

@pytest.mark.skipif(sys.platform.startswith("win"),
                    reason='currently broken')

def test_fromfd_tcp_echo(kernel, portno):
    results = []
    async def server(address):
        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        sock.listen(5)
        results.append('accept wait')
        client, addr = await sock.accept()
        results.append('accept done')
        await spawn(handler, client.detach())
        await sock.close()

    async def handler(fileno):
        client = fromfd(fileno, AF_INET, SOCK_STREAM)
        results.append('handler start')
        while True:
            results.append('recv wait')
            data = await client.recv(100)
            if not data:
                break
            results.append(('handler', data))
            await client.sendall(data)
        results.append('handler done')
        await client.close()

    async def client(address):
        results.append('client start')
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.send(b'Msg1')
        await sleep(0.1)
        resp = await sock.recv(100)
        results.append(('client', resp))
        await sock.send(b'Msg2')
        await sleep(0.1)
        resp = await sock.recv(100)
        results.append(('client', resp))
        results.append('client close')
        await sock.close()

    async def main():
        async with TaskGroup() as g:
            await g.spawn(server, ('', portno))
            await g.spawn(client, ('localhost', portno))

    kernel.run(main())

    assert results == [
        'accept wait',
        'client start',
        'accept done',
        'handler start',
        'recv wait',
        ('handler', b'Msg1'),
        'recv wait',
        ('client', b'Msg1'),
        ('handler', b'Msg2'),
        'recv wait',
        ('client', b'Msg2'),
        'client close',
        'handler done'
    ]


def test_accept_timeout(kernel, portno):
    results = []
    async def server(address):
        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        sock.listen(1)
        results.append('accept wait')
        try:
            client, addr = await timeout_after(0.5, sock.accept())
            results.append('not here')
        except TaskTimeout:
            results.append('accept timeout')
        await sock.close()

    kernel.run(server(('', portno)))

    assert results == [
        'accept wait',
        'accept timeout'
    ]


def test_accept_cancel(kernel, portno):
    results = []
    async def server(address):
        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        sock.listen(1)
        results.append('accept wait')
        try:
            client, addr = await sock.accept()
            results.append('not here')
        except CancelledError:
            results.append('accept cancel')
        await sock.close()

    async def canceller():
        task = await spawn(server, ('', portno))
        await sleep(0.5)
        await task.cancel()

    kernel.run(canceller())
    assert results == [
        'accept wait',
        'accept cancel'
    ]


def test_recv_timeout(kernel):
    results = []
    async def server(address, accepting_event):
        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        sock.listen(1)
        results.append('accept wait')
        await accepting_event.set()
        client, addr = await sock.accept()
        results.append('recv wait')
        try:
            data = await timeout_after(0.5, client.recv(8192))
            results.append('not here')
        except TaskTimeout:
            results.append('recv timeout')
        await client.close()
        await sock.close()

    async def canceller():
        accepting_event = Event()
        task = await spawn(server, ('', 25000), accepting_event)
        await accepting_event.wait()
        sock = socket(AF_INET, SOCK_STREAM)
        results.append('client connect')
        await sock.connect(('localhost', 25000))
        await sleep(1.0)
        await sock.close()
        results.append('client done')
        await task.join()

    kernel.run(canceller())

    assert results == [
        'accept wait',
        'client connect',
        'recv wait',
        'recv timeout',
        'client done'
    ]


def test_recv_cancel(kernel):
    results = []
    async def server(address, accepting_event):
        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        sock.listen(1)
        results.append('accept wait')
        await accepting_event.set()
        client, addr = await sock.accept()
        results.append('recv wait')
        try:
            data = await client.recv(8192)
            results.append('not here')
        except CancelledError:
            results.append('recv cancel')
        await client.close()
        await sock.close()

    async def canceller():
        accepting_event = Event()
        task = await spawn(server, ('', 25000), accepting_event)
        await accepting_event.wait()
        sock = socket(AF_INET, SOCK_STREAM)
        results.append('client connect')
        await sock.connect(('localhost', 25000))
        await sleep(1.0)
        await task.cancel()
        await sock.close()
        results.append('client done')

    kernel.run(canceller())

    assert results == [
        'accept wait',
        'client connect',
        'recv wait',
        'recv cancel',
        'client done'
    ]


def test_recvfrom_timeout(kernel):
    results = []
    async def server(address):
        sock = socket(AF_INET, SOCK_DGRAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        results.append('recvfrom wait')
        try:
            await timeout_after(0.5, sock.recvfrom(8192))
            results.append('not here')
        except TaskTimeout:
            results.append('recvfrom timeout')
        await sock.close()

    async def canceller():
        t = await spawn(server, ('', 25000))
        await sleep(1.0)
        results.append('client done')
        await t.join()

    kernel.run(canceller())

    assert results == [
        'recvfrom wait',
        'recvfrom timeout',
        'client done'
    ]


def test_recvfrom_cancel(kernel):
    results = []
    async def server(address):
        sock = socket(AF_INET, SOCK_DGRAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(address)
        results.append('recvfrom wait')
        try:
            await sock.recvfrom(8192)
            results.append('not here')
        except CancelledError:
            results.append('recvfrom cancel')
        await sock.close()

    async def canceller():
        task = await spawn(server, ('', 25000))
        await sleep(1.0)
        await task.cancel()
        results.append('client done')

    kernel.run(canceller())

    assert results == [
        'recvfrom wait',
        'recvfrom cancel',
        'client done'
    ]


def test_buffer_into(kernel):
    from array import array
    results = []
    async def sender(s1):
        a = array('i', range(1000000))
        await s1.sendall(a)

    async def receiver(s2):
        a = array('i', (0 for n in range(1000000)))
        view = memoryview(a).cast('B')
        total = 0
        while view:
            nrecv = await s2.recv_into(view)
            if not nrecv:
                break
            total += nrecv
            view = view[nrecv:]

        results.append(a)

    s1, s2 = socketpair()

    async def main():
        async with TaskGroup() as g:
            await g.spawn(sender, s1)
            await g.spawn(receiver, s2)

    kernel.run(main())
    s1._socket.close()
    s2._socket.close()

    assert all(n == x for n, x in enumerate(results[0]))


def test_buffer_stream_into(kernel):
    from array import array
    results = []
    async def sender(s1):
        a = array('i', range(1000000))
        await s1.sendall(b'Hello world\n')
        await s1.sendall(a)

    async def receiver(s2):
        a = array('i', (0 for n in range(1000000)))
        view = memoryview(a).cast('B')
        total = 0
        s2_stream = s2.as_stream()
        assert isinstance(s2_stream, io.SocketStream)
        line = await s2_stream.readline()
        assert line == b'Hello world\n'
        while view:
            nrecv = await s2_stream.readinto(view)
            if not nrecv:
                break
            total += nrecv
            view = view[nrecv:]

        results.append(a)

    s1, s2 = socketpair()

    async def main():
        async with TaskGroup() as g:
            await g.spawn(sender, s1)
            await g.spawn(receiver, s2)

    kernel.run(main())
    s1._socket.close()
    s2._socket.close()

    assert all(n == x for n, x in enumerate(results[0]))

@pytest.mark.skipif(sys.platform.startswith("win"),
                    reason='broken on Windows')
def test_buffer_makefile_into(kernel):
    from array import array
    results = []
    async def sender(s1):
        a = array('i', range(1000000))
        await s1.sendall(b'Hello world\n')
        await s1.sendall(a)

    async def receiver(s2):
        a = array('i', (0 for n in range(1000000)))
        view = memoryview(a).cast('B')
        total = 0
        s2_file = s2.makefile('rb')
        assert isinstance(s2_file, io.FileStream)

        line = await s2_file.readline()
        assert line == b'Hello world\n'
        while view:
            nrecv = await s2_file.readinto(view)
            if not nrecv:
                break
            total += nrecv
            view = view[nrecv:]

        results.append(a)

    s1, s2 = socketpair()

    async def main():
        async with TaskGroup() as g:
            await g.spawn(sender, s1)
            await g.spawn(receiver, s2)

    kernel.run(main())
    s1._socket.close()
    s2._socket.close()

    assert all(n == x for n, x in enumerate(results[0]))



def test_read_write_on_same_socket(kernel):
    async def main():
        s1, s2 = socketpair()
        t1 = await spawn(s1.recv, 1000)
        # Large enough send to trigger blocking on write:
        N = 10000000
        t2 = await spawn(s1.sendall, b"x" * N)
        # Above is the actual test -- right now it triggers a crash.
        # Rest of this is just to clean up:
        # Let t1 finish:
        await s2.sendall(b"x")
        # Let t2 finish:
        n = 0
        while n < N:
            n += len(await s2.recv(N))
        await t1.join()
        await t2.join()

    kernel.run(main())

def test_create_connection(kernel):
    evt = Event()
    async def server(addr):
        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(addr)
        sock.listen(1)
        await evt.set()
        client, addr = await sock.accept()
        assert addr[1] == 25001
        await client.close()
        await sock.close()

    async def client(servaddr, clientaddr):
        await evt.wait()
        sock = await create_connection(servaddr, source_address=clientaddr)
        await sock.close()

    async def main():
        async with TaskGroup() as g:
            await g.spawn(server, ('', 25000))
            await sleep(0.1)
            await g.spawn(client, ('localhost', 25000), ('localhost', 25001))
        assert not any(g.exceptions)

    kernel.run(main)

def test_create_bad_connection(kernel):
    async def main():
        with pytest.raises(OSError):
            await create_connection(('python.org', 1))

    kernel.run(main)

# Smoke test on various socket functions
@pytest.mark.skipif(True,
                    reason='unreliable')
def test_socket_funcs(kernel):
    async def main():
        r = await getaddrinfo('', 80)
        r = await getfqdn('')
        r = await gethostbyname('127.0.0.1')
        r = await gethostbyname_ex('127.0.0.1')
        r = await gethostname()
        r = await gethostbyaddr('127.0.0.1')
        r = await getnameinfo(('127.0.0.1', 80), NI_NUMERICHOST)
    kernel.run(main)




from curio import run, tcp_server
from curio import ssl
from socket import *


KEYFILE = "ssl_test_rsa"    # Private key
CERTFILE = "ssl_test.crt"   # Certificate (self-signed)


async def echo_handler(client, addr):
    print('Connection from', addr)
    try:
        client.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
    except (OSError, NameError):
        pass
    s = client.as_stream()
    while True:
        data = await s.read(102400)
        if not data:
            break
        await s.write(data)
    await s.close()
    print('Connection closed')


if __name__ == '__main__':
    ssl_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
    ssl_context.load_cert_chain(certfile=CERTFILE, keyfile=KEYFILE)
    run(tcp_server, '', 25000, echo_handler, ssl=ssl_context)



# happy.py
# An implementation of RFC 6555 (Happy Eyeballs).
# See: https://tools.ietf.org/html/rfc6555

from curio import socket, TaskGroup, ignore_after, run
import itertools

async def open_tcp_stream(hostname, port, delay=0.3):
    # Get all of the possible targets for a given host/port
    targets = await socket.getaddrinfo(hostname, port, type=socket.SOCK_STREAM)
    if not targets:
        raise OSError(f'nothing known about {hostname}:{port}')

    # Cluster the targets into unique address families (e.g., AF_INET, AF_INET6, etc.)
    # and make sure the first entries are from a different family.
    families = [ list(g) for _, g in itertools.groupby(targets, key=lambda t: t[0]) ]
    targets = [ fam.pop(0) for fam in families ]
    targets.extend(itertools.chain(*families))

    # List of accumulated errors to report in case of total failure
    errors = []

    # Task group to manage a collection concurrent tasks.
    # Cancels all remaining once an interesting result is returned.
    async with TaskGroup(wait=object) as group:

        # Attempt to make a connection request
        async def try_connect(sockargs, addr, errors):
            sock = socket.socket(*sockargs)
            try:
                await sock.connect(addr)
                return sock
            except Exception as e:
                await sock.close()
                errors.append(e)

       # Walk the list of targets and try connections with a staggered delay
        for *sockargs, _, addr in targets:
            await group.spawn(try_connect, sockargs, addr, errors)
            async with ignore_after(delay):
                task = await group.next_done()
                if not task.exception:
                    group.completed = task
                    break

    if group.completed:
        return group.completed.result
    else:
        raise OSError(errors)


async def main():
    result = await open_tcp_stream('www.python.org', 80)
    print(result)

if __name__ == '__main__':
    run(main)












# dualserve.py
#
# An example of a server implementation that works both in Curio and
# as a normal threaded application

import threading
from curio import run, spawn_thread, AWAIT

# This is a normal synchronous function.  It is used in both synchronous
# and asynchronous code.

def echo_handler(client, addr):
    print('Connection from', addr)
    with client:
        while True:
            data = AWAIT(client.recv, 100000)
            if not data:
                break
            AWAIT(client.sendall, b'Got:' + data)
    print('Connection closed')

# A Traditional threaded server
def threaded_echo_server(addr):
    import socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
    sock.bind(addr)
    sock.listen(5)
    print('Threaded server running on:', addr)
    while True:
        client, addr = sock.accept()
        threading.Thread(target=echo_handler, args=(client, addr), daemon=True).start()

# An async server
async def async_echo_server(addr):
    import curio.socket as socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
    sock.bind(addr)
    sock.listen(5)
    print('Async server running on:', addr)
    while True:
        client, addr = await sock.accept()
        await spawn_thread(echo_handler, client, addr, daemon=True)

if __name__ == '__main__':
    threading.Thread(target=threaded_echo_server, args=(('',25000),)).start()
    run(async_echo_server, ('',26000))






# test_file.py

import os.path
from curio import *
import pytest

dirname = os.path.dirname(__file__)
testinput = os.path.join(dirname, 'testdata.txt')


def test_read(kernel):
    async def main():
        async with aopen(testinput, 'r') as f:
            data = await f.read()
            assert f.closed == False

        assert data == 'line 1\nline 2\nline 3\n'

    kernel.run(main())


def test_readall(kernel):
    async def main():
        async with aopen(testinput, 'r') as f:
            data = await f.readall()
            assert f.closed == False
        assert data == 'line 1\nline 2\nline 3\n'

    kernel.run(main())


def test_read1(kernel):
    async def main():
        async with aopen(testinput, 'rb') as f:
            data = await f.read1(1000)
        with open(testinput, 'rb') as f:
            data2 = f.read1(1000)
        assert data == data2

    kernel.run(main())


def test_readinto(kernel):
    async def main():
        async with aopen(testinput, 'rb') as f:
            buf = bytearray(1000)
            n = await f.readinto(buf)

        with open(testinput, 'rb') as f:
            buf2 = bytearray(1000)
            n2 = f.readinto(buf2)

        assert buf[:n] == buf2[:n2]

    kernel.run(main())


def test_readinto1(kernel):
    async def main():
        async with aopen(testinput, 'rb') as f:
            buf = bytearray(1000)
            n = await f.readinto1(buf)
        with open(testinput, 'rb') as f:
            buf2 = bytearray(1000)
            n2 = f.readinto1(buf2)
        assert buf[:n] == buf2[:n]

    kernel.run(main())


def test_readline(kernel):
    async def main():
        async with aopen(testinput, 'r') as f:
            lines = []
            while True:
                line = await f.readline()
                if not line:
                    break
                lines.append(line)

        assert lines == ['line 1\n', 'line 2\n', 'line 3\n']

    kernel.run(main())


def test_readlines(kernel):
    async def main():
        async with aopen(testinput, 'r') as f:
            lines = await f.readlines()

        assert lines == ['line 1\n', 'line 2\n', 'line 3\n']

    kernel.run(main())


def test_readiter(kernel):
    async def main():
        async with aopen(testinput, 'r') as f:
            lines = []
            async for line in f:
                lines.append(line)

        assert lines == ['line 1\n', 'line 2\n', 'line 3\n']

    kernel.run(main())

def test_read_anext(kernel):
    async def main():
        async with aopen(testinput, 'r') as f:
            lines = []
            while True:
                line = await anext(f, '')
                if not line:
                    break
                lines.append(line)

        assert lines == ['line 1\n', 'line 2\n', 'line 3\n']

    kernel.run(main())

def test_read_anext2(kernel):
    async def main():
        async with aopen(testinput, 'r') as f:
            lines = []
            with pytest.raises(StopAsyncIteration):
                while True:
                    line = await anext(f)
                    lines.append(line)

        assert lines == ['line 1\n', 'line 2\n', 'line 3\n']

    kernel.run(main())

def test_bad_usage(kernel):
    async def main():
        f = aopen(testinput, 'r')
        with pytest.raises(RuntimeError):
            await f.read()

wlines = ['line1\n', 'line2\n', 'line3\n']


def test_write(kernel, tmpdir):
    async def main():
        outname = tmpdir.join('tmp.txt')
        async with aopen(outname, 'w') as f:
            outdata = ''.join(wlines)
            await f.write(outdata)
            await f.flush()

        assert open(outname).read() == outdata

    kernel.run(main())


def test_writelines(kernel, tmpdir):
    async def main():
        outname = tmpdir.join('tmp.txt')
        async with aopen(outname, 'w') as f:
            await f.writelines(wlines)

        assert open(outname).readlines() == wlines

    kernel.run(main())


def test_seek_tell(kernel):
    async def main():
        async with aopen(testinput, 'rb') as f:
            await f.seek(10)
            n = await f.tell()
            assert n == 10
            data = await f.read()

        with open(testinput, 'rb') as f:
            f.seek(10)
            n2 = f.tell()
            assert n2 == 10
            data2 = f.read()

        assert data == data2

    kernel.run(main())

def test_truncate(kernel, tmpdir):
    async def main():
        outname = tmpdir.join('tmp.txt')
        async with aopen(outname, 'wb') as f:
            await f.write(b'12345')
            await f.flush()
            assert await f.tell() == 5
            await f.truncate(2)
            await f.seek(2)
            await f.write(b'6789')
            await f.close()
        async with aopen(outname, 'rb') as f:
            data = await f.read()
            assert data == b'126789'

    kernel.run(main())

def test_sync_iter(kernel):
    async def main():
        async with aopen(testinput, 'r') as f:
            try:
                for line in f:
                    pass

                assert False, 'sync-iteration should have failed'
            except SyncIOError:
                assert True

    kernel.run(main())


def test_sync_with(kernel):
    async def main():
        f = aopen(testinput, 'r')
        try:
            with f:
                pass
            assert False, 'sync-with should have failed'
        except AsyncOnlyError:
            assert True

    kernel.run(main())

def test_must_be_asynccontext(kernel):
    async def main():
        f = aopen(testinput, 'r')
        with pytest.raises(RuntimeError):
            data = await f.read()

    kernel.run(main())

def test_blocking(kernel):
    async def main():
        async with aopen(testinput, 'r') as f:
            with f.blocking() as sync_f:
                data = sync_f.read()

        assert data == 'line 1\nline 2\nline 3\n'

    kernel.run(main())

def test_file_misc(kernel):
    async def main():
        f = aopen(testinput, 'r')
        repr(f)
        with pytest.raises(SyncIOError):
            next(f)

    kernel.run(main())



# zmq pull client example.  Requires zmq_pusher.py to be running

import curio_zmq as zmq

async def puller(address):
    ctx = zmq.Context()
    sock = ctx.socket(zmq.PULL)
    sock.connect(address)
    while True:
        msg = await sock.recv()
        if msg == b'exit':
            break
        print('Got:', msg)

if __name__ == '__main__':
    zmq.run(puller, 'tcp://localhost:9000')



# test_thread.py

import pytest
from curio import *
from curio.thread import AWAIT, spawn_thread, is_async_thread
from curio.file import aopen
import time
import pytest


def simple_func(x, y):
    assert is_async_thread()
    AWAIT(sleep(0.25))     # Execute a blocking operation
    AWAIT(sleep, 0.25)     # Alternative
    return x + y

async def simple_coro(x, y):
    await sleep(0.5)
    return x + y

def test_good_result(kernel):
    async def main():
        t = await spawn_thread(simple_func, 2, 3)
        result = await t.join()
        assert result == 5
        assert t.result == 5
        assert t.exception is None

    kernel.run(main)

def test_bad_result(kernel):
    async def main():
        t = await spawn_thread(simple_func, 2, '3')
        try:
            result = await t.join()
            assert False
        except TaskError as e:
            assert isinstance(e.__cause__, TypeError)
            assert True
        else:
            assert False

        try:
            result = await t.result
            assert False
        except TypeError as e:
            assert True
        else:
            assert False

    kernel.run(main)

def test_cancel_result(kernel):
    async def main():
        t = await spawn_thread(simple_func, 2, 3)
        await sleep(0.25)
        await t.cancel()
        try:
            result = await t.join()
            assert False
        except TaskError as e:
            assert isinstance(e.__cause__, TaskCancelled)
            assert True
    kernel.run(main)

def test_thread_good_result(kernel):
    def coro():
        result = AWAIT(simple_coro(2, 3))
        return result

    async def main():
        t = await spawn_thread(coro)
        result = await t.join()
        assert result == 5

    kernel.run(main)

def test_thread_bad_result(kernel):
    def coro():
        with pytest.raises(TypeError):
            result = AWAIT(simple_coro(2, '3'))

    async def main():
        t = await spawn_thread(coro)
        await t.join()

    kernel.run(main)

def test_thread_cancel_result(kernel):
    def func():
        with pytest.raises(TaskCancelled):
            result = AWAIT(simple_coro(2, 3))

    async def main():
        t = await spawn_thread(func)
        await sleep(0.25)
        await t.cancel()

    kernel.run(main)

def test_thread_sync(kernel):
    results = []
    def func(lock):
        with lock:
            results.append('func')

    async def main():
        lock = Lock()
        async with lock:
            results.append('main')
            t = await spawn_thread(func, lock)
            await sleep(0.5)
            results.append('main done')
        await t.join()

    kernel.run(main())
    assert results == [ 'main', 'main done', 'func' ]


def test_thread_timeout(kernel):

    def func():
        with pytest.raises(TaskTimeout):
            with timeout_after(1):
                AWAIT(sleep(2))

    async def main():
        t = await spawn_thread(func)
        await t.join()

    kernel.run(main)


def test_thread_disable_cancellation(kernel):
    def func():
        with disable_cancellation():
            AWAIT(sleep(1))
            assert True

            with enable_cancellation():
                AWAIT(sleep(2))

            assert isinstance(AWAIT(check_cancellation()), TaskTimeout)

        with pytest.raises(TaskTimeout):
            AWAIT(sleep(2))

    async def main():
        t = await spawn_thread(func)
        await sleep(0.5)
        await t.cancel()

    kernel.run(main)

import os
dirname = os.path.dirname(__file__)
testinput = os.path.join(dirname, 'testdata.txt')

def test_thread_read(kernel):
    def func():
        with aopen(testinput, 'r') as f:
            data = AWAIT(f.read())
            assert f.closed == False

        assert data == 'line 1\nline 2\nline 3\n'

    async def main():
        t = await spawn_thread(func)
        await t.join()

    kernel.run(main)

import curio.subprocess as subprocess
import sys

@pytest.mark.skipif(sys.platform.startswith('win'),
                    reason='Broken on windows')
def test_subprocess_popen(kernel):
    def func():
        with subprocess.Popen([sys.executable, '-c', 'print("hello")'], stdout=subprocess.PIPE) as p:
            data = AWAIT(p.stdout.read())
            assert data == b'hello\n'

    async def main():
        t = await spawn_thread(func)
        await t.join()

    kernel.run(main)

def test_task_group_thread(kernel):
    results = []
    async def add(x, y):
        return x + y

    def task():
        task1 = AWAIT(spawn(add, 1, 1))
        task2 = AWAIT(spawn(add, 2, 2))
        task3 = AWAIT(spawn(add, 3, 3))
        w = TaskGroup([task1, task2, task3])
        with w:
            for task in w:
                result = AWAIT(task.join())
                results.append(result)

    async def main():
        t = await spawn_thread(task)
        await t.join()

    kernel.run(main)
    assert results == [2, 4, 6]

def test_task_group_spawn_thread(kernel):
    def add(x, y):
        return x + y

    async def task():
        async with TaskGroup(wait=all) as w:
            await w.spawn_thread(add, 1, 1)
            await w.spawn_thread(add, 2, 2)
            t3 = await w.spawn_thread(add, 3, 3)
            r3 = await t3.join()
            assert r3 == 6

        assert w.results == [2, 4]

    kernel.run(task)

@pytest.mark.skip
def test_thread_context(kernel):
    results = []
    async def func(x, y):
        async with spawn_thread():
            time.sleep(0.5)
            results.append(AWAIT(simple_coro(x, y)))

    async def main():
        t = await spawn(func, 2, 3)
        await sleep(0.1)
        results.append(1)
        await t.join()

    kernel.run(main)
    assert results == [1, 5]

def test_await_passthrough(kernel):
    import time
    def add(x, y):
        AWAIT(time.sleep(0.1))
        AWAIT(time.sleep, 0.1)
        return x + y
    async def main():
        t = await spawn_thread(add, 2, 3)
        await t.wait()
        assert t.result == 5
    kernel.run(main)

def test_errors(kernel):
    # spawn_thread used on a coroutine
    async def main():
        with pytest.raises(TypeError):
            t = await spawn_thread(simple_coro, 2, 3)

    kernel.run(main)

    # AWAIT used on coroutine outside of async-thread
    with pytest.raises(AsyncOnlyError):
        AWAIT(simple_coro(2,3))

    # Premature result
    async def f():
        t = await spawn_thread(simple_func, 2, 3)
        assert t.state != 'TERMINATED'
        with pytest.raises(RuntimeError):
            r = t.result
        with pytest.raises(RuntimeError):
            e = t.exception

    kernel.run(f)

    # Launching a thread with no target
    async def g():
        from curio.thread import AsyncThread
        t = AsyncThread()
        with pytest.raises(RuntimeError):
            await t.start()

    kernel.run(g)





from curio import meta
from curio import *
from functools import partial
import pytest
import sys
import inspect

def test_iscoroutinefunc():
    async def spam(x, y):
        pass

    assert meta.iscoroutinefunction(partial(spam, 1))

def test_instantiate_coroutine():
    async def coro(x, y):
        pass

    def func(x, y):
        pass

    c = meta.instantiate_coroutine(coro(2,3))
    assert inspect.iscoroutine(c)

    d = meta.instantiate_coroutine(coro, 2, 3)
    assert inspect.iscoroutine(d)

    with pytest.raises(TypeError):
        meta.instantiate_coroutine(func(2,3))

    with pytest.raises(TypeError):
        meta.instantiate_coroutine(func, 2, 3)


def test_bad_awaitable():
    def spam(x, y):
        pass

    with pytest.raises(TypeError):
        @meta.awaitable(spam)
        def spam(x, y, z):
            pass


def test_awaitable_partial(kernel):
    def func(x, y, z):
        assert False

    @meta.awaitable(func)
    async def func(x, y, z):
        assert x == 1
        assert y == 2
        assert z == 3
        return True

    async def main():
        assert await func(1, 2, 3)
        assert await ignore_after(1, func(1,2,3))
        assert await ignore_after(1, func, 1, 2, 3)
        assert await ignore_after(1, partial(func, 1, 2), 3)
        assert await ignore_after(1, partial(func, z=3), 1, 2)
        assert await ignore_after(1, partial(partial(func, 1), 2), 3)

        # Try spawns
        t = await spawn(func(1,2,3))
        assert await t.join()

        t = await spawn(func, 1, 2, 3)
        assert await t.join()

        t = await spawn(partial(func, 1, 2), 3)
        assert await t.join()

        t = await spawn(partial(func, z=3), 1, 2)
        assert await t.join()

        t = await spawn(partial(partial(func, 1), 2), 3)
        assert await t.join()


    kernel.run(main)
    kernel.run(func, 1, 2, 3)
    kernel.run(partial(func, 1, 2), 3)
    kernel.run(partial(func, z=3), 1, 2)

if sys.version_info >= (3,7):
    import contextlib
    def test_asynccontextmanager(kernel):
        results = []
        @contextlib.asynccontextmanager
        async def manager():
             try:
                 yield (await coro())
             finally:
                 await cleanup()

        async def coro():
            results.append('coro')
            return 'result'

        async def cleanup():
            results.append('cleanup')

        async def main():
            async with manager() as r:
                results.append(r)

        kernel.run(main)
        assert results == ['coro', 'result', 'cleanup']


    def test_missing_asynccontextmanager(kernel):
        results = []
        async def manager():
             try:
                 yield (await coro())
             finally:
                 await cleanup()

        async def coro():
            results.append('coro')
            return 'result'

        async def cleanup():
            results.append('cleanup')

        async def main():
             async for x in manager():
                  break

        kernel.run(main)
        assert results == ['coro']



# zmq push example. Run the zmq_puller.py program for the client

import curio_zmq as zmq

async def pusher(address):
    ctx = zmq.Context()
    sock = ctx.socket(zmq.PUSH)
    sock.bind(address)
    for n in range(100):
        await sock.send(b'Message %d' % n)
    await sock.send(b'exit')

if __name__ == '__main__':
    zmq.run(pusher, 'tcp://*:9000')



# test_asyncgen.py

import pytest
from curio import *
from curio.meta import finalize, awaitable


# Test to make sure a simple async generator runs
def test_simple_agen(kernel):
    async def countdown(n):
        while n > 0:
            yield n
            n -= 1

    async def main():
        nums = [ n async for n in countdown(5) ]
        assert nums == [5, 4, 3, 2, 1]

    kernel.run(main())

# Test to make sure a simple finally clause executes
def test_simple_agen_final(kernel):
    results = []
    async def countdown(n):
        try:
            while n > 0:
                yield n
                n -= 1
        finally:
            results.append('done')

    async def main():
        nums = [ n async for n in countdown(5) ]
        assert nums == [5, 4, 3, 2, 1]

    kernel.run(main())
    assert results == ['done']

# Make sure application of finalize() works
def test_agen_final_finalize(kernel):
    async def countdown(n):
        try:
            while n > 0:
                yield n
                n -= 1
        finally:
            await sleep(0.0)

    async def main():
        async with finalize(countdown(5)) as c:
            nums = [n async for n in c]
            assert nums == [5, 4, 3, 2, 1]

    kernel.run(main())

# Make sure a try-except without asyncs works
def test_agen_except_ok(kernel):
    async def countdown(n):
        while n > 0:
             try:
                 yield n
             except Exception:
                 pass
             n -= 1

    async def main():
        nums = [n async for n in countdown(5) ]
        assert nums == [5, 4, 3, 2, 1]

    kernel.run(main())

# Test to make sure a simple async generator runs
def test_awaitable_agen(kernel):
    async def countdown(n):
        while n > 0:
             try:
                 yield n
             except Exception:
                 pass
             n -= 1

    def add(x, y):
        return x + y

    @awaitable(add)
    async def add(x, y):
        return x + y

    async def main():
        nums = [ await add(n,n) async for n in countdown(5) ]
        assert nums == [10, 8, 6, 4, 2]

    kernel.run(main())

    nums = [ add(n,n) for n in range(5,0,-1) ]
    assert nums == [10, 8, 6, 4, 2]



# test_sync.py
#
# Different test scenarios designed to run under management of a kernel

from collections import deque
from curio import *
import pytest
import threading
import time
import asyncio

# ---- Synchronization primitives


class TestEvent:

    def test_event_get_wait(self, kernel):
        results = []
        async def event_setter(evt, seconds):
            results.append('sleep')
            await sleep(seconds)
            results.append('event_set')
            await evt.set()

        async def event_waiter(evt):
            results.append('wait_start')
            results.append(evt.is_set())
            await evt.wait()
            results.append('wait_done')
            results.append(evt.is_set())
            evt.clear()
            results.append(evt.is_set())

        async def main():
            evt = Event()
            t1 = await spawn(event_waiter, evt)
            t2 = await spawn(event_setter, evt, 1)
            await t1.join()
            await t2.join()

        kernel.run(main())
        assert results == [
            'wait_start',
            False,
            'sleep',
            'event_set',
            'wait_done',
            True,
            False
        ]

    def test_event_get_immediate(self, kernel):
        results = []
        async def event_setter(evt):
            results.append('event_set')
            await evt.set()

        async def event_waiter(evt, seconds):
            results.append('sleep')
            await sleep(seconds)
            results.append('wait_start')
            await evt.wait()
            results.append('wait_done')

        async def main():
            evt = Event()
            t1 = await spawn(event_waiter, evt, 1)
            t2 = await spawn(event_setter, evt)
            await t1.join()
            await t2.join()

        kernel.run(main())
        assert results == [
            'sleep',
            'event_set',
            'wait_start',
            'wait_done',
        ]

    def test_event_wait_cancel(self, kernel):
        results = []
        async def event_waiter(evt):
            results.append('event_wait')
            try:
                await evt.wait()
            except CancelledError:
                results.append('event_cancel')

        async def event_cancel(seconds):
            evt = Event()
            task = await spawn(event_waiter, evt)
            results.append('sleep')
            await sleep(seconds)
            results.append('cancel_start')
            await task.cancel()
            results.append('cancel_done')

        kernel.run(event_cancel(1))

        assert results == [
            'sleep',
            'event_wait',
            'cancel_start',
            'event_cancel',
            'cancel_done',
        ]

    def test_event_wait_timeout(self, kernel):
        results = []
        async def event_waiter(evt):
            results.append('event_wait')
            try:
                await timeout_after(0.5, evt.wait())
            except TaskTimeout:
                results.append('event_timeout')

        async def event_run(seconds):
            evt = Event()
            task = await spawn(event_waiter, evt)
            results.append('sleep')
            await sleep(seconds)
            results.append('sleep_done')
            await task.join()

        kernel.run(event_run(1))

        assert results == [
            'sleep',
            'event_wait',
            'event_timeout',
            'sleep_done',
        ]

    def test_event_wait_notimeout(self, kernel):
        results = []
        async def event_waiter(evt):
            results.append('event_wait')
            try:
                await timeout_after(1.0, evt.wait())
                results.append('got event')
            except TaskTimeout:
                results.append('event_timeout')

            evt.clear()
            try:
                await evt.wait()
                results.append('got event')
            except TaskTimeout:
                results.append('bad timeout')

        async def event_run():
            evt = Event()
            task = await spawn(event_waiter, evt)
            results.append('sleep')
            await sleep(0.25)
            results.append('event_set')
            await evt.set()
            await sleep(1.0)
            results.append('event_set')
            await evt.set()
            await task.join()

        kernel.run(event_run())
        assert results == [
            'sleep',
            'event_wait',
            'event_set',
            'got event',
            'event_set',
            'got event'
        ]



class TestLock:

    def test_lock_sequence(self, kernel):
        results = []
        async def worker(lck, label):
            results.append(label + ' wait')
            results.append(lck.locked())
            async with lck:
                results.append(label + ' acquire')
                await sleep(0.25)
            results.append(label + ' release')

        async def main():
            lck = Lock()
            t1 = await spawn(worker, lck, 'work1')
            t2 = await spawn(worker, lck, 'work2')
            t3 = await spawn(worker, lck, 'work3')
            await t1.join()
            await t2.join()
            await t3.join()

        kernel.run(main())
        assert results == [
            'work1 wait',
            False,
            'work1 acquire',
            'work2 wait',
            True,
            'work3 wait',
            True,
            'work1 release',
            'work2 acquire',
            'work2 release',
            'work3 acquire',
            'work3 release',
        ]

    def test_lock_acquire_cancel(self, kernel):
        results = []
        async def worker(lck):
            results.append('lock_wait')
            try:
                async with lck:
                    results.append('never here')
            except CancelledError:
                results.append('lock_cancel')

        async def worker_cancel(seconds):
            lck = Lock()
            async with lck:
                task = await spawn(worker, lck)
                results.append('sleep')
                await sleep(seconds)
                results.append('cancel_start')
                await task.cancel()
                results.append('cancel_done')

        kernel.run(worker_cancel(1))

        assert results == [
            'sleep',
            'lock_wait',
            'cancel_start',
            'lock_cancel',
            'cancel_done',
        ]

    def test_lock_acquire_timeout(self, kernel):
        results = []
        async def worker(lck):
            results.append('lock_wait')
            try:
                await timeout_after(0.5, lck.acquire())
                results.append('never here')
                await lck.release()
            except TaskTimeout:
                results.append('lock_timeout')

        async def worker_timeout(seconds):
            lck = Lock()
            async with lck:
                w = await spawn(worker, lck)
                results.append('sleep')
                await sleep(seconds)
                results.append('sleep_done')
                await w.join()

        kernel.run(worker_timeout(1))

        assert results == [
            'sleep',
            'lock_wait',
            'lock_timeout',
            'sleep_done',
        ]


class TestRLock:

    def test_rlock_reenter(self, kernel):
        results = []

        async def inner(lck, label):
            results.append(lck.locked())
            async with lck:
                results.append(label + ' inner acquired')
                results.append(label + ' inner releasing')

        async def worker(lck, label):
            results.append(lck.locked())
            results.append(label + ' wait')
            async with lck:
                results.append(label + ' acquired')
                await sleep(0.25)
                await inner(lck, label)
                results.append(label + ' releasing')

        async def worker_simple(lck):
            results.append('simple wait')
            async with lck:
                results.append('simple acquired')
                results.append('simple releasing')

        async def main():
            lck = RLock()
            t1 = await spawn(worker, lck, 'work1')
            t2 = await spawn(worker, lck, 'work2')
            t3 = await spawn(worker_simple, lck)
            await t1.join()
            await t2.join()
            await t3.join()

        kernel.run(main())
        assert results == [
            False,
            'work1 wait',
            'work1 acquired',
            True,
            'work2 wait',
            'simple wait',
            True,
            'work1 inner acquired',
            'work1 inner releasing',
            'work1 releasing',
            'work2 acquired',
            True,
            'work2 inner acquired',
            'work2 inner releasing',
            'work2 releasing',
            'simple acquired',
            'simple releasing'
        ]

    def test_rlock_notowner(self, kernel):
        async def child1(lck):
            await lck.acquire()
            await sleep(0.25)
            await lck.release()

        async def child2(lck):
            await sleep(0.1)
            with pytest.raises(RuntimeError):
                await lck.release()

        async def main():
            lck = RLock()
            with pytest.raises(RuntimeError):
                await lck.release()

            t1 = await spawn(child1, lck)
            t2 = await spawn(child2, lck)
            await t1.join()
            await t2.join()

        kernel.run(main)


class TestSemaphore:

    def test_sema_sequence(self, kernel):
        results = []
        async def worker(sema, label):
            results.append(label + ' wait')
            results.append(sema.locked())
            async with sema:
                assert sema.value == 0
                results.append(label + ' acquire')
                await sleep(0.25)
            results.append(label + ' release')

        async def main():
            sema = Semaphore()
            t1 = await spawn(worker, sema, 'work1')
            t2 = await spawn(worker, sema, 'work2')
            t3 = await spawn(worker, sema, 'work3')
            await t1.join()
            await t2.join()
            await t3.join()

        kernel.run(main())

        assert results == [
            'work1 wait',
            False,
            'work1 acquire',
            'work2 wait',
            True,
            'work3 wait',
            True,
            'work1 release',
            'work2 acquire',
            'work2 release',
            'work3 acquire',
            'work3 release',
        ]

    def test_sema_sequence2(self, kernel):
        results = []
        async def worker(sema, label, seconds):
            results.append(label + ' wait')
            results.append(sema.locked())
            async with sema:
                results.append(label + ' acquire')
                await sleep(seconds)
            results.append(label + ' release')

        async def main():
            sema = Semaphore(2)
            t1 = await spawn(worker, sema, 'work1', 0.25)
            t2 = await spawn(worker, sema, 'work2', 0.30)
            t3 = await spawn(worker, sema, 'work3', 0.35)
            await t1.join()
            await t2.join()
            await t3.join()

        kernel.run(main())
        assert results == [
            'work1 wait',            # Both work1 and work2 admitted
            False,
            'work1 acquire',
            'work2 wait',
            False,
            'work2 acquire',
            'work3 wait',
            True,
            'work1 release',
            'work3 acquire',
            'work2 release',
            'work3 release',
        ]

    def test_sema_acquire_cancel(self, kernel):
        results = []
        async def worker(lck):
            results.append('lock_wait')
            try:
                async with lck:
                    results.append('never here')
            except CancelledError:
                results.append('lock_cancel')

        async def worker_cancel(seconds):
            lck = Semaphore()
            async with lck:
                task = await spawn(worker, lck)
                results.append('sleep')
                await sleep(seconds)
                results.append('cancel_start')
                await task.cancel()
                results.append('cancel_done')

        kernel.run(worker_cancel(1))

        assert results == [
            'sleep',
            'lock_wait',
            'cancel_start',
            'lock_cancel',
            'cancel_done',
        ]

    def test_sema_acquire_timeout(self, kernel):
        results = []
        async def worker(lck):
            results.append('lock_wait')
            try:
                await timeout_after(0.5, lck.acquire())
                results.append('never here')
                await lck.release()
            except TaskTimeout:
                results.append('lock_timeout')

        async def worker_timeout(seconds):
            lck = Semaphore()
            async with lck:
                w = await spawn(worker, lck)
                results.append('sleep')
                await sleep(seconds)
                results.append('sleep_done')
                await w.join()

        kernel.run(worker_timeout(1))

        assert results == [
            'sleep',
            'lock_wait',
            'lock_timeout',
            'sleep_done',
        ]


class TestCondition:

    def test_cond_sequence(self, kernel):
        results = []
        async def consumer(cond, q, label):
            while True:
                async with cond:
                    if not q:
                        results.append(label + ' wait')
                        await cond.wait()
                    item = q.popleft()
                    if item is None:
                        break
                    results.append((label, item))
            results.append(label + ' done')

        async def producer(cond, q, count, nproducers):
            for n in range(count):
                async with cond:
                    q.append(n)
                    results.append(('producing', n))
                    await cond.notify()
                await sleep(0.1)

            for n in range(nproducers):
                async with cond:
                    q.append(None)
                    results.append(('ending', n))
                    await cond.notify()
                await sleep(0.1)

        async def main():
            cond = Condition(Lock())
            q = deque()
            t1 = await spawn(consumer, cond, q, 'cons1')
            t2 = await spawn(consumer, cond, q, 'cons2')
            t3 = await spawn(producer, cond, q, 4, 2)
            await t1.join()
            await t2.join()
            await t3.join()

        kernel.run(main())

        assert results == [
            'cons1 wait',
            'cons2 wait',
            ('producing', 0),
            ('cons1', 0),
            'cons1 wait',
            ('producing', 1),
            ('cons2', 1),
            'cons2 wait',
            ('producing', 2),
            ('cons1', 2),
            'cons1 wait',
            ('producing', 3),
            ('cons2', 3),
            'cons2 wait',
            ('ending', 0),
            ('cons1 done'),
            ('ending', 1),
            ('cons2 done')
        ]

    def test_cond_wait_cancel(self, kernel):
        results = []
        async def worker(cond):
            try:
                async with cond:
                    results.append('cond_wait')
                    await cond.wait()
                    results.append('never here')
            except CancelledError:
                results.append('worker_cancel')

        async def worker_cancel(seconds):
            cond = Condition()
            task = await spawn(worker, cond)
            results.append('sleep')
            await sleep(seconds)
            results.append('cancel_start')
            await task.cancel()
            results.append('cancel_done')

        kernel.run(worker_cancel(1))

        assert results == [
            'sleep',
            'cond_wait',
            'cancel_start',
            'worker_cancel',
            'cancel_done',
        ]

    def test_cond_wait_timeout(self, kernel):
        results = []
        async def worker(cond):
            try:
                async with cond:
                    results.append('cond_wait')
                    await timeout_after(0.25, cond.wait())
                    results.append('never here')
            except TaskTimeout:
                results.append('worker_timeout')

        async def worker_cancel(seconds):
            cond = Condition()
            task = await spawn(worker, cond)
            results.append('sleep')
            await sleep(seconds)
            results.append('done')
            await task.join()

        kernel.run(worker_cancel(1))

        assert results == [
            'sleep',
            'cond_wait',
            'worker_timeout',
            'done'
        ]

    def test_cond_notify_all(self, kernel):
        results = []
        async def worker(cond):
            async with cond:
                results.append('cond_wait')
                await cond.wait()
                results.append('wait_done')

        async def worker_notify(seconds):
            cond = Condition()
            t1 = await spawn(worker, cond)
            t2 = await spawn(worker, cond)
            t3 = await spawn(worker, cond)
            results.append('sleep')
            await sleep(seconds)
            async with cond:
                results.append('notify')
                await cond.notify_all()
            results.append('done')
            await t1.join()
            await t2.join()
            await t3.join()

        kernel.run(worker_notify(1))

        assert results == [
            'sleep',
            'cond_wait',
            'cond_wait',
            'cond_wait',
            'notify',
            'done',
            'wait_done',
            'wait_done',
            'wait_done',
        ]

    def test_cond_waitfor(self, kernel):
        results = []
        async def consumer(cond, q, label):
            async with cond:
                results.append(label + ' waitfor')
                await cond.wait_for(lambda: len(q) > 2)
                results.append((label, len(q)))
            results.append(label + ' done')

        async def producer(cond, q, count):
            for n in range(count):
                async with cond:
                    q.append(n)
                    results.append(('producing', n))
                    await cond.notify()
                await sleep(0.1)

        async def main():
            cond = Condition()
            q = deque()
            t1 = await spawn(consumer, cond, q, 'cons1')
            t2 = await spawn(consumer, cond, q, 'cons2')
            t3 = await spawn(producer, cond, q, 4)
            await t1.join()
            await t2.join()
            await t3.join()

        kernel.run(main())
        assert results == [
            'cons1 waitfor',
            'cons2 waitfor',
            ('producing', 0),
            ('producing', 1),
            ('producing', 2),
            ('cons1', 3),
            'cons1 done',
            ('producing', 3),
            ('cons2', 4),
            'cons2 done'
        ]

    def test_condition_error(self, kernel):
        async def main():
            c = Condition()
            with pytest.raises(RuntimeError):
                await c.notify()

            with pytest.raises(RuntimeError):
                await c.wait()

        kernel.run(main)

class TestUniversalEvent:

    def test_uevent_get_wait(self, kernel):
        results = []
        async def event_setter(evt, seconds):
            results.append('sleep')
            await sleep(seconds)
            results.append('event_set')
            await evt.set()

        async def event_waiter(evt):
            results.append('wait_start')
            results.append(evt.is_set())
            await evt.wait()
            results.append('wait_done')
            results.append(evt.is_set())
            evt.clear()
            results.append(evt.is_set())

        async def main():
            evt = UniversalEvent()
            t1 = await spawn(event_waiter, evt)
            t2 = await spawn(event_setter, evt, 1)
            await t1.join()
            await t2.join()

        kernel.run(main())
        assert results == [
            'wait_start',
            False,
            'sleep',
            'event_set',
            'wait_done',
            True,
            False
        ]


    def test_uevent_get_twait(self, kernel):
        results = []
        async def event_setter(evt, seconds):
            results.append('sleep')
            await sleep(seconds)
            results.append('event_set')
            await evt.set()

        def event_waiter(evt):
            results.append('wait_start')
            results.append(evt.is_set())
            evt.wait()
            results.append('wait_done')
            results.append(evt.is_set())
            evt.clear()
            results.append(evt.is_set())

        async def main():
            evt = UniversalEvent()
            t1 = threading.Thread(target=event_waiter, args=(evt,))
            t1.start()
            t2 = await spawn(event_setter, evt, 1)
            await run_in_thread(t1.join)
            await t2.join()

        kernel.run(main())
        assert results == [
            'wait_start',
            False,
            'sleep',
            'event_set',
            'wait_done',
            True,
            False
        ]

    def test_uevent_get_asyncio_set(self, kernel):
        results = []
        async def event_setter(evt, seconds):
            results.append('sleep')
            await asyncio.sleep(seconds)
            results.append('event_set')
            await evt.set()

        async def event_waiter(evt):
            results.append('wait_start')
            results.append(evt.is_set())
            await evt.wait()
            results.append('wait_done')
            results.append(evt.is_set())
            evt.clear()
            results.append(evt.is_set())

        async def main():
            evt = UniversalEvent()
            t1 = await spawn(event_waiter, evt)
            await sleep(0.05)
            t2 = threading.Thread(target=asyncio.run, args=[event_setter(evt, 1)])
            t2.start()
            await t1.join()
            await run_in_thread(t2.join)

        kernel.run(main())
        assert results == [
            'wait_start',
            False,
            'sleep',
            'event_set',
            'wait_done',
            True,
            False
        ]


    def test_uevent_get_asyncio_wait(self, kernel):
        results = []
        async def event_setter(evt, seconds):
            results.append('sleep')
            await sleep(seconds)
            results.append('event_set')
            await evt.set()

        async def event_waiter(evt):
            results.append('wait_start')
            results.append(evt.is_set())
            await evt.wait()
            results.append('wait_done')
            results.append(evt.is_set())
            evt.clear()
            results.append(evt.is_set())

        async def main():
            evt = UniversalEvent()
            t1 = threading.Thread(target=asyncio.run, args=[event_waiter(evt)])
            t1.start()
            await sleep(0.1)
            t2 = await spawn(event_setter, evt, 1)
            await run_in_thread(t1.join)
            await t2.join()

        kernel.run(main())
        assert results == [
            'wait_start',
            False,
            'sleep',
            'event_set',
            'wait_done',
            True,
            False
        ]

class TestResult:
    def test_value(self, kernel):
        
        async def work(x, y, r):
            await r.set_value(x+y)

        async def main():
            r = Result()
            await spawn(work, 2, 3, r)
            assert await r.unwrap() == 5

        kernel.run(main)

    def test_error(self, kernel):
        async def work(x, y, r):
            try:
                await r.set_value(x+y)
            except Exception as err:
                await r.set_exception(err)

        async def main():
            r = Result()
            await spawn(work, 2, "3", r)
            with pytest.raises(TypeError):
                await r.unwrap()

        kernel.run(main)

        
class TestUniversalResult:
    def test_universal_value(self, kernel):
        
        def work(x, y, r):
            r.set_value(x+y)

        async def main(r1, r2):
            value = await r1.unwrap()
            await r2.set_value(value)

        r1 = UniversalResult()
        r2 = UniversalResult()
        r3 = UniversalResult()
        threading.Thread(target=work, args=[2,3,r1]).start()
        threading.Thread(target=asyncio.run, args=[main(r1, r2)]).start()
        kernel.run(main, r2, r3)
        assert r3.unwrap() == 5

    def test_universal_error(self, kernel):
        
        def work(x, y, r):
            try:
                r.set_value(x+y)
            except Exception as err:
                r.set_exception(err)

        async def main(r1, r2):
            try:
                value = await r1.unwrap()
                await r2.set_value(value)
            except Exception as err:
                await r2.set_exception(err)

        r1 = UniversalResult()
        r2 = UniversalResult()
        r3 = UniversalResult()
        threading.Thread(target=work, args=[2,"3",r1]).start()
        threading.Thread(target=asyncio.run, args=[main(r1, r2)]).start()
        kernel.run(main, r2, r3)
        with pytest.raises(TypeError):
            val = r3.unwrap()


def test_repr():
    # For test coverage
    for cls in [Lock, Event, Semaphore, Condition, RLock, UniversalEvent, Result, UniversalResult ]:
        repr(cls())



# Example: A simple echo server written using asyncio streams

import asyncio
import ssl


KEYFILE = "ssl_test_rsa"    # Private key
CERTFILE = "ssl_test.crt"   # Certificate (self-signed)


async def echo_client(reader, writer):
    addr = writer.get_extra_info('peername')
    print('Connection from', addr)
    while True:
        data = await reader.read(100000)
        if not data:
            break
        writer.write(data)
        await writer.drain()
    print('Connection closed')


if __name__ == '__main__':
    context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
    context.load_cert_chain(certfile=CERTFILE, keyfile=KEYFILE)

    # import uvloop
    # asyncio.set_event_loop(uvloop.new_event_loop())

    loop = asyncio.get_event_loop()
    coro = asyncio.start_server(
        echo_client, '127.0.0.1', 25000, loop=loop, ssl=context)
    loop.run_until_complete(coro)
    loop.run_forever()



# Example of implementing a Promise class using
# built-in synchronization primitives.

from curio import Event

class Promise:
    def __init__(self):
        self._event = Event()
        self._data = None
        self._exception = None

    def __repr__(self):
        res = super().__repr__()
        if self.is_set():
            extra = repr(self._exception) if self._exception else repr(self._data)
        else:
            extra = 'unset'
        return f'<{res[1:-1]} [{extra}]>'

    def is_set(self):
        '''Return `True` if the promise is set'''
        return self._event.is_set()

    def clear(self):
        '''Clear the promise'''
        self._data = None
        self._exception = None
        self._event.clear()

    async def set(self, data):
        '''Set the promise. Wake all waiting tasks (if any).'''
        self._data = data
        await self._event.set()

    async def get(self):
        '''Wait for the promise to be set, and return the data.

        If an exception was set, it will be raised.'''
        await self._event.wait()

        if self._exception is not None:
            raise self._exception

        return self._data

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        if exc_type is not None:
            self._exception = exc
            await self._event.set()

            return True

async def consumer(promise):
    return await promise.get()

def test_promise(kernel):
    async def producer(promise):
        await promise.set(42)

    async def main():
        promise = Promise()
        assert not promise.is_set()

        producer_task = await curio.spawn(producer(promise))

        assert 42 == await consumer(promise)
        assert promise.is_set()
        await producer_task.join()

    kernel.run(main())

def test_promise_exception(kernel):
    async def exception_producer(promise):
        async with promise:
            raise RuntimeError()

    async def main():
        promise = Promise()
        producer_task = await curio.spawn(exception_producer(promise))

        with pytest.raises(RuntimeError):
            await consumer(promise)

        await producer_task.join()

    kernel.run(main())

def test_promise_no_exception(kernel):
    async def main():
        promise = Promise()
        async with promise:
            pass
        assert not promise.is_set()

    kernel.run(main())

def test_promise_internals(kernel):
    async def main():
        promise = Promise()
        assert not promise.is_set()
        assert repr(promise).endswith('[unset]>')

        await promise.set(42)

        assert promise.is_set()
        assert promise._data == 42
        assert promise._exception is None
        assert repr(promise).endswith('[42]>')

        promise.clear()

        assert not promise.is_set()
        assert promise._data is None
        assert promise._exception is None
        assert repr(promise).endswith('[unset]>')

        async with promise:
            raise RuntimeError()

        assert promise.is_set()
        assert promise._data is None
        assert isinstance(promise._exception, RuntimeError)
        assert repr(promise).endswith('[RuntimeError()]>')

        promise.clear()

        assert not promise.is_set()
        assert promise._data is None
        assert promise._exception is None
        assert repr(promise).endswith('[unset]>')

    kernel.run(main())



# boundsema.py
#
# Curio can often be extended to implement more specialized forms
# of basic concurrency primitives.  For example, here is how
# you could implement a bound semaphore.

from curio import Semaphore

class BoundedSemaphore(Semaphore):

    def __init__(self, value=1):
        self._bound = value
        super().__init__(value)

    @property
    def bound(self):
        return self._bound

    async def release(self):
        if self._value >= self._bound:
            raise ValueError('BoundedSemaphore released too many times')
        await super().release()




# zmq rpc example.

import curio_zmq as zmq

async def rpc_server(address):
    ctx = zmq.Context()
    sock = ctx.socket(zmq.REP)
    sock.bind(address)
    while True:
        func, args, kwargs = await sock.recv_pyobj()
        try:
            result = func(*args, **kwargs)
            await sock.send_pyobj(result)
        except Exception as e:
            await sock.send_pyobj(e)

if __name__ == '__main__':
    zmq.run(rpc_server, 'tcp://*:9000')



import socket
import sys
import pytest
from curio import Kernel
from curio.monitor import Monitor
from curio.debug import *

@pytest.fixture(scope='session')
def kernel(request):
    k = Kernel(debug=[longblock, logcrash])
    m = Monitor(k)
    request.addfinalizer(lambda: k.run(shutdown=True))
    request.addfinalizer(m.close)
    return k


# This is based on https://unix.stackexchange.com/a/132524
@pytest.fixture(scope='function')
def portno():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('', 0))
    _, port = s.getsockname()
    s.close()
    return port

collect_ignore = []
if sys.version_info < (3,6):
    collect_ignore.append("test_asyncgen.py")




# signal_handling.py
#
# This example illustrates how you might handle a Unix signal from Curio.
# The basic idea is that you need to install a standard signal handler
# using Python's built-in signal module.  That handler then communciates
# with a Curio task using a UniversalEvent (or a UniversalQueue).

import signal
import os
import curio

signal_evt = curio.UniversalEvent()

# Ordinary Python signal handler.
def sig_handler(signo, frame):
    signal_evt.set()

# A Curio task waiting for an event
async def coro():
    print("Waiting....")
    await signal_evt.wait()
    print("Got a signal!")

# Set up and execution
def main():
    signal.signal(signal.SIGHUP, sig_handler)
    print("Send me a SIGHUP", os.getpid())
    curio.run(coro)

if __name__ == '__main__':
    main()





from curio.timequeue import TimeQueue

def test_timequeue_expired():
    q = TimeQueue()

    delta = q.next_deadline(10)
    assert delta == None

    q.push('a', 50)
    q.push('b', 25)
    q.push('c', 100)
    # this should return the number of seconds to the next deadline
    delta = q.next_deadline(5)
    assert delta == 20
    items = list(q.expired(25))
    assert items == [(25, 'b')]

    items = list(q.expired(101))
    assert items == [(50, 'a'), (100, 'c')]




# Example: A simple echo server written directly with sockets

from curio import run, spawn
from curio.socket import *


async def echo_server(address):
    sock = socket(AF_INET, SOCK_STREAM)
    sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
    sock.bind(address)
    sock.listen(5)
    print('Server listening at', address)
    async with sock:
        while True:
            client, addr = await sock.accept()
            print('Connection from', addr)
            await spawn(echo_client, client)


async def echo_client(client):
    async with client:
        while True:
            data = await client.recv(10000)
            if not data:
                break
            await client.sendall(data)
    print('Connection closed')


if __name__ == '__main__':
    run(echo_server, ('', 25000))



# test_channel.py

import pytest
from socket import *
from curio.channel import Connection, Channel, AuthenticationError
from curio.io import SocketStream
from curio import spawn, sleep, CancelledError, TaskTimeout, timeout_after, TaskError, TaskGroup
import copy
import sys

@pytest.fixture
def conns():
    sock1, sock2 = socketpair()
    sock1_s = SocketStream(sock1)
    sock2_s = SocketStream(sock2)
    c1 = Connection(sock1_s, sock1_s)
    c2 = Connection(sock2_s, sock2_s)
    return (c1, c2)

def test_connection_hello(kernel, conns):
    results = []

    async def server(c):
        async with c:
            await c.send('server hello world')
            results.append(await c.recv())

    async def client(c):
        async with c:
            msg = await c.recv()
            results.append(msg)
            await c.send('client hello world')

    async def main(c1, c2):
        async with TaskGroup() as g:
            await g.spawn(server, c1)
            await g.spawn(client, c2)

    kernel.run(main(*conns))
    assert results == ['server hello world',
                       'client hello world']


def test_connection_hello_bytes(kernel, conns):
    results = []

    async def server(c):
        async with c:
            await c.send(b'server hello world')
            results.append(await c.recv())

    async def client(c):
        async with c:
            msg = await c.recv()
            results.append(msg)
            await c.send(b'client hello world')

    async def main(c1, c2):
        async with TaskGroup() as g:
            await g.spawn(server, c1)
            await g.spawn(client, c2)

    kernel.run(main(*conns))
    assert results == [b'server hello world',
                       b'client hello world']


def test_connection_large(kernel, conns):
    results = []
    data = list(range(1000000))

    async def server(c):
        async with c:
            await c.send(data)
            results.append(await c.recv())

    async def client(c):
        async with c:
            msg = await c.recv()
            results.append(msg)
            await c.send(len(msg))

    async def main(c1, c2):
        async with TaskGroup() as g:
            await g.spawn(server, c1)
            await g.spawn(client, c2)

    kernel.run(main(*conns))
    assert results == [data,
                       len(data)]


def test_connection_auth(kernel, conns):
    results = []

    async def server(c):
        async with c:
            await c.authenticate_server(b'peekaboo')
            await c.send('server hello world')
            results.append(await c.recv())

    async def client(c):
        async with c:
            await c.authenticate_client(b'peekaboo')
            msg = await c.recv()
            results.append(msg)
            await c.send('client hello world')

    async def main(c1, c2):
        async with TaskGroup() as g:
            await g.spawn(server, c1)
            await g.spawn(client, c2)

    kernel.run(main(*conns))

    assert results == ['server hello world',
                       'client hello world']



def test_connection_auth_fail(kernel, conns):
    async def server(c):
        async with c:
            with pytest.raises(AuthenticationError):
                await c.authenticate_server(b'peekaboo')

    async def client(c):
        async with c:
            with pytest.raises(AuthenticationError):
                await c.authenticate_client(b'what?')

    async def main(c1, c2):
        async with TaskGroup() as g:
            await g.spawn(server, c1)
            await g.spawn(client, c2)

    kernel.run(main(*conns))


def test_connection_send_partial_bytes(kernel, conns):
    results = []
    data = b'abcdefghijklmnopqrstuvwxyz'

    async def server(c):
        async with c:
            await c.send_bytes(data, offset=5, size=10)
            results.append(await c.recv())
            await c.send_bytes(data, offset=5)
            results.append(await c.recv())
            await c.send_bytes(data, size=10)
            results.append(await c.recv())

            # Try some bad inputs
            try:
                await c.send_bytes(data, offset=50)
            except ValueError as e:
                results.append(str(e))

            try:
                await c.send_bytes(data, size=50)
            except ValueError as e:
                results.append(str(e))

            try:
                await c.send_bytes(data, offset=-10)
            except ValueError as e:
                results.append(str(e))

            try:
                await c.send_bytes(data, size=-10)
            except ValueError as e:
                results.append(str(e))

    async def client(c):
        async with c:
            msg = await c.recv_bytes()
            results.append(msg)
            await c.send(len(msg))
            msg = await c.recv_bytes()
            results.append(msg)
            await c.send(len(msg))
            msg = await c.recv_bytes()
            results.append(msg)
            await c.send(len(msg))

    async def main(c1, c2):
        async with TaskGroup() as g:
            await g.spawn(server, c1)
            await g.spawn(client, c2)

    kernel.run(main(*conns))
    assert results == [data[5:15], 10,
                       data[5:], len(data[5:]),
                       data[:10], 10,
                       'buffer length < offset',
                       'buffer length < offset + size',
                       'offset is negative',
                       'size is negative',

                       ]

@pytest.mark.skipif(sys.platform.startswith("win"),
                    reason="not supported on Windows")
def test_connection_from_connection(kernel):
    import multiprocessing
    p1, p2 = multiprocessing.Pipe()
    c1 = Connection.from_Connection(p1)
    c2 = Connection.from_Connection(p2)

    results = []

    async def server(c):
        async with c:
            await c.send('server hello world')
            results.append(await c.recv())

    async def client(c):
        async with c:
            msg = await c.recv()
            results.append(msg)
            await c.send('client hello world')

    async def main(c1, c2):
        async with TaskGroup() as g:
            await g.spawn(server, c1)
            await g.spawn(client, c2)

    kernel.run(main(c1, c2))
    assert results == ['server hello world',
                       'client hello world']


def test_connection_recv_cancel(kernel, conns):
    results = []

    async def client(c):
        async with c:
            try:
                msg = await c.recv()
                results.append(msg)
            except CancelledError:
                results.append('cancel')

    async def main(c):
        task = await spawn(client, c)
        await sleep(1)
        await task.cancel()
        results.append('done cancel')

    c1, c2 = conns
    kernel.run(main(c2))
    assert results == ['cancel', 'done cancel']


def test_connection_recv_timeout(kernel, conns):
    results = []

    async def client(c):
        try:
            msg = await timeout_after(1.0, c.recv())
            results.append(msg)
        except TaskTimeout:
            results.append('timeout')

    async def main(c):
        task = await spawn(client, c)
        await task.join()
        results.append('done')

    c1, c2 = conns
    kernel.run(main(c2))
    assert results == ['timeout', 'done']


def test_connection_send_cancel(kernel, conns):
    results = []

    async def client(c):
        async with c:
            try:
                msg = 'x' * 10000000   # Should be large enough to cause send blocking
                await c.send(msg)
                await c.send(msg)  # Send twice to get blocking
                results.append('success')
            except CancelledError:
                results.append('cancel')

    async def main(c):
        task = await spawn(client, c)
        await sleep(1)
        await task.cancel()
        results.append('done cancel')

    c1, c2 = conns
    kernel.run(main(c2))
    assert results == ['cancel', 'done cancel']


def test_connection_send_timeout(kernel, conns):
    results = []

    async def client(c):
        try:
            msg = 'x' * 10000000
            await timeout_after(1, c.send(msg))
            await timeout_after(1, c.send(msg))
            results.append('success')
        except TaskTimeout:
            results.append('timeout')

    async def main(c):
        task = await spawn(client, c)
        await task.join()
        results.append('done')

    c1, c2 = conns
    kernel.run(main(c2))
    assert results == ['timeout', 'done']


@pytest.fixture
def chs():
    ch1 = Channel(('localhost', 0))
    ch1.bind()
    ch2 = copy.deepcopy(ch1)
    return ch1, ch2

def test_channel_hello(kernel, chs):
    results = []

    async def server(ch):
        c = await ch.accept()
        async with c:
            await c.send('server hello world')
            results.append(await c.recv())

    async def client(ch):
        c = await ch.connect()
        async with c:
            msg = await c.recv()
            results.append(msg)
            await c.send('client hello world')

    async def main(ch1, ch2):
        async with TaskGroup() as g:
            await g.spawn(server, ch1)
            await g.spawn(client, ch2)

    kernel.run(main(*chs))
    assert results == ['server hello world',
                       'client hello world']


def test_channel_hello_auth(kernel, chs):
    results = []

    async def server(ch):
        c = await ch.accept(authkey=b'peekaboo')
        async with c:
            await c.send('server hello world')
            results.append(await c.recv())

    async def client(ch):
        c = await ch.connect(authkey=b'peekaboo')
        async with c:
            msg = await c.recv()
            results.append(msg)
            await c.send('client hello world')

    async def main(ch1, ch2):
        async with TaskGroup() as g:
            await g.spawn(server, ch1)
            await g.spawn(client, ch2)

    kernel.run(main(*chs))
    assert results == ['server hello world',
                       'client hello world']


def test_channel_hello_auth_fail(kernel, chs):

    async def server(ch):
        c = await ch.accept(authkey=b'peekaboo')
        async with c:
            await c.send('server hello world')

    async def client(ch):
        with pytest.raises(AuthenticationError):
            c = await ch.connect(authkey=b'what?')

    async def main(ch1, ch2):
        t1 = await spawn(server, ch1)
        t2 = await spawn(client, ch2)
        await t2.join()
        await t1.cancel()

    kernel.run(main(*chs))

@pytest.mark.skip
def test_channel_slow_connect(kernel, chs):
    results = []

    async def server(ch):
        await sleep(2)
        c = await ch.accept(authkey=b'peekaboo')
        async with c:
            await c.send('server hello world')
            results.append(await c.recv())

    async def client(ch):
        c = await ch.connect(authkey=b'peekaboo')
        async with c:
            msg = await c.recv()
            results.append(msg)
            await c.send('client hello world')

    async def main(ch1, ch2):
        async with ch1, ch2:
            t1 = await spawn(server, ch1)
            t2 = await spawn(client, ch2)
            await t1.join()
            await t2.join()

    kernel.run(main(*chs))
    assert results == ['server hello world',
                       'client hello world']


def test_recv_bytes_into(kernel, chs):
    import array
    results = { }
    async def client(ch):
        c = await ch.connect(authkey=b'peekaboo')
        a = array.array('i', [0]*100000)
        nrecv = await c.recv_bytes_into(a)
        results['nrecv'] = nrecv
        results['got'] = a
        await c.close()

    async def server(serv):
        c = await serv.accept(authkey=b'peekaboo')
        a = array.array('i', range(100000))
        nsent = await c.send_bytes(a)
        results['nsent'] = nsent
        results['sent'] = a
        await c.close()

    async def main(ch1, ch2):
        t1 = await spawn(server(ch1))
        t2 = await spawn(client(ch2))
        await t1.join()
        await t2.join()

    kernel.run(main, *chs)
    assert results['nrecv'] == results['nsent']
    assert results['got'] == results['sent']



# test_io.py

from curio import *
from curio.socket import *
import io
import socket as std_socket
import pytest
import sys

def test_socket_blocking(kernel, portno):
    '''
    Test of exposing a socket in blocking mode
    '''
    done = Event()

    results = []

    def sync_client(sock):
        assert isinstance(sock, std_socket.socket)
        data = sock.recv(8192)
        results.append(('client', data))
        sock.sendall(data)

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        with client.blocking() as s:
            await run_in_thread(sync_client, s)
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Message')
        data = await sock.recv(8192)
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()


    kernel.run(main())

    assert results == [
        'handler start',
        ('client', b'Message'),
        'handler done'
    ]

@pytest.mark.skipif(sys.platform.startswith("win"),
                    reason="not supported on Windows")
def test_socketstream_blocking(kernel, portno):
    '''
    Test of exposing a socket stream in blocking mode
    '''
    done = Event()
    results = []

    def sync_client(f):
        assert isinstance(f, io.RawIOBase)
        data = f.read(8192)
        results.append(('client', data))
        f.write(data)

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        s = client.as_stream()
        with s.blocking() as f:
            await run_in_thread(sync_client, f)
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Message')
        data = await sock.recv(8192)
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        ('client', b'Message'),
        'handler done'
    ]

@pytest.mark.skipif(sys.platform.startswith("win"),
                    reason="not supported on Windows")
def test_filestream_blocking(kernel, portno):
    '''
    Test of exposing a socket in blocking mode
    '''
    done = Event()
    results = []

    def sync_client(f):
        assert isinstance(f, io.RawIOBase)
        data = f.read(8192)
        results.append(('client', data))
        f.write(data)

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        s = client.makefile('rwb', buffering=0)
        with s.blocking() as f:
            await run_in_thread(sync_client, f)
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Message')
        data = await sock.recv(8192)
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        ('client', b'Message'),
        'handler done'
    ]

@pytest.mark.skipif(sys.platform.startswith("win"),
                    reason="not supported on Windows")
def test_filestream_bad_blocking(kernel, portno):
    '''
    Test of exposing a socket in blocking mode with buffered data error
    '''
    done = Event()
    results = []

    async def handler(client, addr):
        await client.send(b'OK')
        s = client.makefile('rwb', buffering=0)
        line = await s.readline()
        assert line == b'hello\n'
        try:
            with s.blocking() as f:
                pass
        except IOError:
            results.append(True)
        else:
            results.append(False)
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'hello\nworld\n')
        data = await sock.recv(8192)
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())
    assert results[0]

@pytest.mark.skipif(sys.platform.startswith("win"),
                    reason="not supported on Windows")
def test_socketstream_bad_blocking(kernel, portno):
    '''
    Test of exposing a socket in blocking mode with buffered data error
    '''
    done = Event()
    results = []

    async def handler(client, addr):
        await client.send(b'OK')
        s = client.as_stream()
        line = await s.readline()
        assert line == b'hello\n'
        try:
            with s.blocking() as f:
                pass
        except IOError:
            results.append(True)
        else:
            results.append(False)
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'hello\nworld\n')
        data = await sock.recv(8192)
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())
    assert results[0]

def test_read_partial(kernel, portno):
    done = Event()
    results = []
    async def handler(client, addr):
        try:
            await client.send(b'OK')
            s = client.as_stream()
            line = await s.readline()
            results.append(line)
            data = await s.read(2)
            results.append(data)
            data = await s.read(1000)
            results.append(data)
        finally:
            await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'hello\nworld\n')
        await sock.close()


    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())
    assert results == [ b'hello\n', b'wo', b'rld\n']

def test_readall(kernel, portno):
    done = Event()
    results = []

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        async with client.as_stream() as s:
            data = await s.readall()
            results.append(data)
            results.append('handler done')
            await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Msg1\n')
        await sleep(0.1)
        await sock.send(b'Msg2\n')
        await sleep(0.1)
        await sock.send(b'Msg3\n')
        await sleep(0.1)
        await sock.close()


    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        b'Msg1\nMsg2\nMsg3\n',
        'handler done'
    ]


def test_readall_partial(kernel, portno):
    done = Event()

    async def handler(client, addr):
        await client.send(b'OK')
        s = client.as_stream()
        line = await s.readline()
        assert line == b'hello\n'
        data = await s.readall()
        assert data == b'world\n'
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'hello\nworld\n')
        await sock.close()


    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())


def test_readall_timeout(kernel, portno):
    done = Event()
    results = []

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        s = client.as_stream()
        try:
            data = await timeout_after(0.5, s.readall())
        except TaskTimeout as e:
            results.append(e.bytes_read)
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Msg1\n')
        await sleep(1)
        await sock.send(b'Msg2\n')
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        b'Msg1\n',
        'handler done'
    ]


def test_read_exactly(kernel, portno):
    done = Event()
    results = []

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        s = client.as_stream()
        for n in range(3):
            results.append(await s.read_exactly(5))
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Msg1\nMsg2\nMsg3\n')
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        b'Msg1\n',
        b'Msg2\n',
        b'Msg3\n',
        'handler done'
    ]


def test_read_exactly_incomplete(kernel, portno):
    done = Event()
    results = []
    async def handler(client, addr):
        await client.send(b'OK')
        s = client.as_stream()
        try:
            await s.read_exactly(100)
        except EOFError as e:
            results.append(e.bytes_read)
        finally:
            await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Msg1\nMsg2\nMsg3\n')
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results[0] ==  b'Msg1\nMsg2\nMsg3\n'

def test_read_exactly_timeout(kernel, portno):
    done = Event()
    results = []

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        s = client.as_stream()
        try:
            data = await timeout_after(0.5, s.read_exactly(10))
            results.append(data)
        except TaskTimeout as e:
            results.append(e.bytes_read)
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Msg1\n')
        await sleep(1)
        await sock.send(b'Msg2\n')
        await sock.close()


    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        b'Msg1\n',
        'handler done'
    ]



def test_readline(kernel, portno):
    done = Event()
    results = []

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        s = client.as_stream()
        for n in range(3):
            results.append(await s.readline())
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Msg1\nMsg2\nMsg3\n')
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        b'Msg1\n',
        b'Msg2\n',
        b'Msg3\n',
        'handler done'
    ]


def test_readlines(kernel, portno):
    done = Event()
    results = []

    async def handler(client, addr):
        await client.send(b'OK')
        results.append('handler start')
        s = client.as_stream()
        results.extend(await s.readlines())
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Msg1\nMsg2\nMsg3\n')
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        b'Msg1\n',
        b'Msg2\n',
        b'Msg3\n',
        'handler done'
    ]


def test_readlines_timeout(kernel, portno):
    done = Event()
    results = []

    async def handler(client, addr):
        await client.send(b'OK')
        results.append('handler start')
        s = client.as_stream()
        try:
            await timeout_after(0.5, s.readlines())
        except TaskTimeout as e:
            results.extend(e.lines_read)
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Msg1\nMsg2\n')
        await sleep(1)
        await sock.send(b'Msg3\n')
        await sock.close()


    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        b'Msg1\n',
        b'Msg2\n',
        'handler done'
    ]


def test_writelines(kernel, portno):
    done = Event()
    results = []

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        s = client.as_stream()
        results.append(await s.readall())
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        s = sock.as_stream()
        await s.writelines([b'Msg1\n', b'Msg2\n', b'Msg3\n'])
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [
        'handler start',
        b'Msg1\nMsg2\nMsg3\n',
        'handler done'
    ]


def test_writelines_timeout(kernel, portno):
    done = Event()
    results = []
    async def handler(client, addr):
        await client.send(b'OK')
        s = client.as_stream()
        await sleep(1)
        results.append(await s.readall())
        await done.set()

    def line_generator():
        n = 0
        while True:
            yield b'Msg%d\n' % n
            n += 1

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        s = sock.as_stream()
        try:
            await timeout_after(0.5, s.writelines(line_generator()))
        except TaskTimeout as e:
            results.append(e.bytes_written)
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results[0] == len(results[1])

@pytest.mark.skipif(sys.platform.startswith("win"),
                    reason="fails on windows")
def test_write_timeout(kernel, portno):
    done = Event()
    results = []
    async def handler(client, addr):
        await client.send(b'OK')
        s = client.as_stream()
        await sleep(1)
        results.append(await s.readall())
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        s = sock.as_stream()
        try:
            msg = b'x'*10000000  # Must be big enough to fill buffers
            await timeout_after(0.5, s.write(msg))
        except TaskTimeout as e:
            results.append(e.bytes_written)
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results[0] == len(results[1])

def test_iterline(kernel, portno):
    results = []

    async def handler(client, addr):
        results.append('handler start')
        await client.send(b'OK')
        s = client.as_stream()
        async for line in s:
            results.append(line)
        results.append('handler done')

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Msg1\nMsg2\nMsg3\n')
        await sock.close()
        await serv.cancel()

    async def main():
        async with TaskGroup() as g:
            serv = await g.spawn(tcp_server, '', portno, handler)
            await g.spawn(test_client, ('localhost', portno), serv)

    kernel.run(main())

    assert results == [
        'handler start',
        b'Msg1\n',
        b'Msg2\n',
        b'Msg3\n',
        'handler done'
    ]


def test_double_recv(kernel, portno):
    done = Event()
    results = []

    async def bad_handler(client):
        results.append('bad handler')
        await sleep(0.1)
        try:
            await client.recv(1000)   # <- This needs to fail. Task already reading on the socket
            results.append('why am I here?')
        except ReadResourceBusy as e:
            results.append('good handler')

    async def handler(client, addr):
        results.append('handler start')
        await spawn(bad_handler, client, daemon=True)
        await client.send(b'OK')
        data = await client.recv(1000)
        results.append(data)
        results.append('handler done')
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sleep(1)
        await sock.send(b'Msg')
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        client = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await client.join()

    kernel.run(main())

    assert results == [
        'handler start',
        'bad handler',
        'good handler',
        b'Msg',
        'handler done'
        ]

@pytest.mark.skipif(True,
                    reason="flaky")
def test_sendall_cancel(kernel, portno):
    done = Event()
    start = Event()
    results = {}

    async def handler(client, addr):
        await start.wait()
        nrecv = 0
        while True:
            data = await client.recv(1000000)
            if not data:
                break
            nrecv += len(data)
        results['handler'] = nrecv
        await client.close()
        await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        try:
            await sock.sendall(b'x'*10000000)
        except CancelledError as e:
            results['sender'] = e.bytes_sent
        await sock.close()

    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        t = await spawn(test_client, ('localhost', portno), serv)
        await sleep(0.1)
        await t.cancel()
        await start.set()
        await serv.cancel()
        await done.wait()

    kernel.run(main())

    assert results['handler'] == results['sender']


def test_stream_bad_context(kernel, portno):
    done = Event()
    results = []
    async def handler(client, addr):
        await client.send(b'OK')
        try:
            with client.as_stream() as s:
                data = await s.readall()
        except AsyncOnlyError:
            results.append(True)
        finally:
            await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Hello\n')
        await sock.close()


    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [ True ]

def test_stream_bad_iter(kernel, portno):
    done = Event()
    results = []
    async def handler(client, addr):
        await client.send(b'OK')
        try:
            async with client.as_stream() as s:
                for line in s:
                    pass
        except AsyncOnlyError:
            results.append(True)
        finally:
            await done.set()

    async def test_client(address, serv):
        sock = socket(AF_INET, SOCK_STREAM)
        await sock.connect(address)
        await sock.recv(8)
        await sock.send(b'Hello\n')
        await sock.close()


    async def main():
        serv = await spawn(tcp_server, '', portno, handler)
        c = await spawn(test_client, ('localhost', portno), serv)
        await done.wait()
        await serv.cancel()
        await c.join()

    kernel.run(main())

    assert results == [ True ]

def test_io_waiting(kernel):
    async def handler(sock):
        result = await sock.accept()

    async def main():
        from curio.traps import _io_waiting

        sock = socket(AF_INET, SOCK_STREAM)
        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, True)
        sock.bind(('',25000))
        sock.listen(1)
        async with sock:
            t1 = await spawn(handler, sock)
            await sleep(0.1)
            r, w = await _io_waiting(sock)
            assert t1 == r
            assert w == None
            await t1.cancel()

        r,w = await _io_waiting(0)
        assert (r, w) == (None, None)

    kernel.run(main())

def test_io_unregister(kernel):
    # This is purely a code coverage test
    async def reader(sock):
        from curio.traps import _read_wait
        await _read_wait(sock.fileno())

    async def main():
        from curio.traps import _write_wait
        sock = socket(AF_INET, SOCK_DGRAM)
        sock.bind(('', 26000))
        t = await spawn(reader(sock))
        await sleep(0.1)
        await _write_wait(sock.fileno())
        await t.cancel()
        await sock.close()
    kernel.run(main())
    assert True



# test_subprocess.py

import sys
from curio import subprocess
from curio import *
import os
import pytest
import sys

# ---- Test subprocesses and worker task related functions

executable = sys.executable
dirname = os.path.dirname(__file__)


def test_simple(kernel):
    results = []
    async def subproc():
        out = await subprocess.run([executable, os.path.join(dirname, 'child.py')], stdout=subprocess.PIPE)
        results.append(out.stdout)
        results.append(out.returncode)

    kernel.run(subproc())
    assert results == [
        b't-minus 4\nt-minus 3\nt-minus 2\nt-minus 1\n'.replace(b'\n', os.linesep.encode('latin-1')),
        0,
    ]


def test_simple_check_output(kernel):
    results = []
    async def subproc():
        out = await subprocess.check_output([executable, os.path.join(dirname, 'child.py')])
        results.append(out)

    kernel.run(subproc())
    assert results == [
        b't-minus 4\nt-minus 3\nt-minus 2\nt-minus 1\n'.replace(b'\n', os.linesep.encode('latin-1')),
    ]


def test_bad_cmd(kernel):
    results = []
    async def subproc():
        try:
            out = await subprocess.run([executable, '-m', 'curio.test.bad'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
            results.append('what?')
        except subprocess.CalledProcessError:
            results.append('bad command')

    kernel.run(subproc())
    assert results == ['bad command']


def test_bad_cmd_check_output(kernel):
    results = []
    async def subproc():
        try:
            out = await subprocess.check_output([executable, '-m', 'curio.test.bad'], stderr=subprocess.STDOUT)
            results.append('what?')
        except subprocess.CalledProcessError:
            results.append('bad command')

    kernel.run(subproc())
    assert results == ['bad command']


def test_timeout(kernel):
    results = []
    async def subproc():
        try:
            async with timeout_after(0.5):
                out = await subprocess.run([executable, os.path.join(dirname, 'child.py')], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                results.append('what?')
        except TaskTimeout as e:
            results.append('timeout')
            results.append(e.stdout)
            results.append(e.stderr)

    kernel.run(subproc())
    if sys.platform.startswith('win'):
        assert results == ['timeout', b'', b'']
    else:
        assert results == ['timeout', b't-minus 4\n', b'']

def test_universal():
    with pytest.raises(RuntimeError):
        p = subprocess.Popen([executable, '-m', 'bad'], universal_newlines=True)

def test_stdin_pipe(kernel):
    async def main():
         p1 = subprocess.Popen([executable, os.path.join(dirname, 'child.py')], stdout=subprocess.PIPE)
         p2 = subprocess.Popen([executable, os.path.join(dirname, 'ichild.py')], stdin=p1.stdout, stdout=subprocess.PIPE)
         out = await p2.stdout.read()
         assert out == b'4\n'.replace(b'\n', os.linesep.encode('latin-1'))

    kernel.run(main())

def test_check_output_stdin(kernel):
    async def main():
         out = await subprocess.check_output([executable, os.path.join(dirname, 'ichild.py')],
                                             input=b'Line1\nLine2\nLine3\n')
         assert out == b'3\n'.replace(b'\n', os.linesep.encode('latin-1'))

    kernel.run(main())

def test_no_input_cancel(kernel):
    async def child():
        p = subprocess.Popen([executable, os.path.join(dirname, 'child.py')], stdin=subprocess.PIPE)
        try:
            out = await p.communicate(input=b'x'*10000000)
            assert False
        except CancelledError as e:
            assert e.stdout == b''
            assert e.stderr == b''
            raise

    async def main():
        t = await spawn(child)
        await sleep(0.1)
        await t.cancel()

    kernel.run(main())

def test_popen_join(kernel):
    async def main():
         p = subprocess.Popen([executable, '-c', 'import time;time.sleep(1)'])
         code = await p.wait()
         assert code == 0

    kernel.run(main)

def test_io_error(kernel):
    async def main():
         with pytest.raises(BrokenPipeError):
             out = await subprocess.check_output([executable, '-c', 'import sys, time; sys.stdin.close(); time.sleep(1)'],
                                           input=b'x'*10000000)

    kernel.run(main)






# test_queue.py

from collections import deque
from curio import *
import time
import threading
import asyncio
from curio.traps import _read_wait

def test_queue_simple(kernel):
    results = []
    async def consumer(queue, label):
        while True:
            item = await queue.get()
            if item is None:
                break
            results.append((label, item))
            await queue.task_done()
        await queue.task_done()
        results.append(label + ' done')

    async def producer():
        queue = Queue()
        results.append('producer_start')
        c1 = await spawn(consumer(queue, 'cons1'))
        c2 = await spawn(consumer(queue, 'cons2'))
        await sleep(0.1)
        for n in range(4):
            await queue.put(n)
            await sleep(0.1)
        for n in range(2):
            await queue.put(None)
        results.append('producer_join')
        await queue.join()
        results.append('producer_done')
        await c1.join()
        await c2.join()

    kernel.run(producer())

    assert results == [
        'producer_start',
        ('cons1', 0),
        ('cons2', 1),
        ('cons1', 2),
        ('cons2', 3),
        'producer_join',
        'cons1 done',
        'cons2 done',
        'producer_done',
    ]


def test_queue_unbounded(kernel):
    results = []
    async def consumer(queue, label):
        while True:
            item = await queue.get()
            if item is None:
                break
            results.append((label, item))
            await queue.task_done()
        await queue.task_done()
        results.append(label + ' done')

    async def producer():
        queue = Queue()
        results.append('producer_start')
        c1 = await spawn(consumer(queue, 'cons1'))
        await sleep(0.1)
        for n in range(4):
            await queue.put(n)
        await queue.put(None)
        results.append('producer_join')
        await queue.join()
        results.append('producer_done')
        await c1.join()

    kernel.run(producer())

    assert results == [
        'producer_start',
        'producer_join',
        ('cons1', 0),
        ('cons1', 1),
        ('cons1', 2),
        ('cons1', 3),
        'cons1 done',
        'producer_done',
    ]


def test_queue_bounded(kernel):
    results = []
    async def consumer(queue, label):
        while True:
            item = await queue.get()
            if item is None:
                break
            results.append((label, item))
            await sleep(0.1)
            await queue.task_done()
        await queue.task_done()
        results.append(label + ' done')

    async def producer():
        queue = Queue(maxsize=2)
        results.append('producer_start')
        await spawn(consumer(queue, 'cons1'))
        await sleep(0.1)
        for n in range(4):
            await queue.put(n)
            results.append(('produced', n))
        await queue.put(None)
        results.append('producer_join')
        await queue.join()
        results.append('producer_done')

    kernel.run(producer())

    assert results == [
        'producer_start',
        ('produced', 0),
        ('produced', 1),
        ('cons1', 0),
        ('produced', 2),
        ('cons1', 1),
        ('produced', 3),
        ('cons1', 2),
        'producer_join',
        ('cons1', 3),
        'cons1 done',
        'producer_done',
    ]


def test_queue_get_cancel(kernel):
    # Make sure a blocking get can be cancelled
    results = []
    async def consumer():
        queue = Queue()
        try:
            results.append('consumer waiting')
            item = await queue.get()
            results.append('not here')
        except CancelledError:
            results.append('consumer cancelled')

    async def driver():
        task = await spawn(consumer())
        await sleep(0.5)
        await task.cancel()

    kernel.run(driver())
    assert results == [
        'consumer waiting',
        'consumer cancelled'
    ]


def test_queue_put_cancel(kernel):
    # Make sure a blocking put() can be cancelled
    results = []

    async def producer():
        queue = Queue(1)
        results.append('producer_start')
        await queue.put(0)
        try:
            await queue.put(1)
            results.append('not here')
        except CancelledError:
            results.append('producer_cancel')

    async def driver():
        task = await spawn(producer())
        await sleep(0.5)
        await task.cancel()

    kernel.run(driver())
    assert results == [
        'producer_start',
        'producer_cancel'
    ]


def test_queue_get_timeout(kernel):
    # Make sure a blocking get respects timeouts
    results = []
    async def consumer():
        queue = Queue()
        try:
            results.append('consumer waiting')
            item = await timeout_after(0.5, queue.get())
            results.append('not here')
        except TaskTimeout:
            results.append('consumer timeout')

    kernel.run(consumer())
    assert results == [
        'consumer waiting',
        'consumer timeout'
    ]


def test_queue_put_timeout(kernel):
    # Make sure a blocking put() respects timeouts
    results = []

    async def producer():
        queue = Queue(1)
        results.append('producer start')
        await queue.put(0)
        try:
            await timeout_after(0.5, queue.put(1))
            results.append('not here')
        except TaskTimeout:
            results.append('producer timeout')

    kernel.run(producer())
    assert results == [
        'producer start',
        'producer timeout'
    ]

def test_queue_qsize(kernel):
    async def main():
        q = Queue()
        repr(q)
        await q.put(1)
        assert q.qsize() == 1
    kernel.run(main)

def test_priority_queue(kernel):
    results = []
    priorities = [4, 2, 1, 3]

    async def consumer(queue):
        while True:
            item = await queue.get()
            if item[1] is None:
                break
            results.append(item[1])
            await queue.task_done()
            await sleep(0.2)
        await queue.task_done()

    async def producer():
        queue = PriorityQueue()

        for n in priorities:
            await queue.put((n, n))

        await queue.put((10, None))

        await spawn(consumer(queue))

        await queue.join()

    kernel.run(producer())
    assert results == sorted(priorities)


def test_lifo_queue(kernel):
    results = []
    items = range(4)

    async def consumer(queue):
        while True:
            item = await queue.get()
            if item is None:
                break
            results.append(item)
            await queue.task_done()
            await sleep(0.2)
        await queue.task_done()

    async def producer():
        queue = LifoQueue()

        await queue.put(None)

        for n in items:
            await queue.put(n)

        await spawn(consumer(queue))

        await queue.join()

    kernel.run(producer())
    assert results == list(reversed(items))

def test_univ_queue_basic(kernel):
    q = UniversalQueue()
    assert q.empty()
    assert q.qsize() == 0
    assert not q.full()

def test_univ_queue_sync_async(kernel):
    result = [ ]
    async def consumer(q):
        while True:
            item = await q.get()
            if item is None:
                break
            result.append(item)
            await q.task_done()

    def producer(q):
        for i in range(10):
            q.put(i)
            time.sleep(0.1)
        q.join()
        assert True

    async def main():
        q = UniversalQueue()

        t1 = await spawn(consumer(q))
        t2 = threading.Thread(target=producer, args=(q,))
        t2.start()
        await run_in_thread(t2.join)
        await q.put(None)
        await t1.join()
        assert result == [0,1,2,3,4,5,6,7,8,9]

    kernel.run(main())

def test_univ_queue_async_sync(kernel):
    result = []
    def consumer(q):
        while True:
            item = q.get()
            if item is None:
                break
            result.append(item)
            q.task_done()

    async def producer(q):
        for i in range(10):
            await q.put(i)
            await sleep(0.1)
        await q.join()

    async def main():
        q = UniversalQueue()

        t1 = threading.Thread(target=consumer, args=(q,))
        t1.start()
        t2 = await spawn(producer(q))
        await t2.join()
        await q.put(None)
        await run_in_thread(t1.join)
        assert result == [0,1,2,3,4,5,6,7,8,9]

    kernel.run(main())

def test_univ_queue_cancel(kernel):
    result = []

    async def consumer(q):
        while True:
            try:
                item = await timeout_after(0.1, q.get())
            except TaskTimeout:
                continue
            if item is None:
                break
            result.append(item)
            await q.task_done()

    def producer(q):
        for i in range(10):
            q.put(i)
            time.sleep(0.2)
        q.join()

    async def main():
        q = UniversalQueue(maxsize=2)
        t1 = await spawn(consumer(q))
        t2 = threading.Thread(target=producer, args=(q,))
        t2.start()
        await run_in_thread(t2.join)
        await q.put(None)
        await t1.join()
        assert result == [0,1,2,3,4,5,6,7,8,9]

    kernel.run(main())

def test_univ_queue_multiple_consumer(kernel):
    result = []

    async def consumer(q):
        while True:
            item = await q.get()
            if item is None:
                break
            result.append(item)
            await q.task_done()

    def producer(q):
        for i in range(1000):
            q.put(i)
        q.join()

    async def main():
        q = UniversalQueue(maxsize=10)
        t1 = await spawn(consumer(q))
        t2 = await spawn(consumer(q))
        t3 = await spawn(consumer(q))
        t4 = threading.Thread(target=producer, args=(q,))
        t4.start()
        await run_in_thread(t4.join)
        await q.put(None)
        await q.put(None)
        await q.put(None)
        await t1.join()
        await t2.join()
        await t3.join()
        assert list(range(1000)) == sorted(result)

    kernel.run(main())


def test_univ_queue_multiple_kernels(kernel):
    result = []

    async def consumer(q):
        while True:
            item = await q.get()
            if item is None:
                break
            result.append(item)
            await q.task_done()

    def producer(q):
        for i in range(1000):
            q.put(i)
        q.join()

    async def main():
        q = UniversalQueue(maxsize=10)

        t1 = threading.Thread(target=run, args=(consumer(q),))
        t1.start()

        t2 = threading.Thread(target=run, args=(consumer(q),))
        t2.start()

        t3 = threading.Thread(target=run, args=(consumer(q),))
        t3.start()
        t4 = threading.Thread(target=producer, args=(q,))
        t4.start()

        await run_in_thread(t4.join)
        await q.put(None)
        await q.put(None)
        await q.put(None)

        t1.join()
        t2.join()
        t3.join()

        assert list(range(1000)) == sorted(result)

    kernel.run(main())

def test_univ_queue_withfd(kernel):
    result = [ ]
    async def consumer(q):
        while True:
            await _read_wait(q)
            item = await q.get()
            if item is None:
                break
            result.append(item)
            await q.task_done()

    def producer(q):
        for i in range(10):
            q.put(i)
            time.sleep(0.1)
        q.join()
        assert True

    async def main():
        q = UniversalQueue(withfd=True)
        t1 = await spawn(consumer(q))
        t2 = threading.Thread(target=producer, args=(q,))
        t2.start()
        await run_in_thread(t2.join)
        await q.put(None)
        await t1.join()
        assert result == [0,1,2,3,4,5,6,7,8,9]

    kernel.run(main())



def test_uqueue_asyncio_prod(kernel):
    async def consumer():
        queue = UniversalQueue(maxsize=2)
        loop = asyncio.new_event_loop()
        t = threading.Thread(target=loop.run_until_complete, args=(producer(queue),))
        t.start()

        results = []
        while True:
            item = await queue.get()
            await queue.task_done()
            if item is None:
                break
            results.append(item)

        assert results == list(range(10))
        t.join()

    async def producer(queue):
        for n in range(10):
            await queue.put(n)
        await queue.put(None)
        await queue.join()

    kernel.run(consumer())

def test_uqueue_asyncio_consumer(kernel):
    results = []
    async def consumer(queue):
        while True:
            item = await queue.get()
            await queue.task_done()
            if item is None:
                break
            results.append(item)

    async def producer(queue):
        for n in range(10):
            await queue.put(n)
        await queue.put(None)
        await queue.join()

    queue = UniversalQueue(maxsize=2)
    t1 = threading.Thread(target=asyncio.run, args=[consumer(queue)])
    t1.start()
    kernel.run(producer, queue)
    t1.join()
    assert results == list(range(10))

def test_uqueue_withfd_corner(kernel):
    async def main():
        queue = UniversalQueue(withfd=True)
        await queue.put(1)
        queue._get_sock.recv(1000)    # Drain the socket
        item = await queue.get()
        assert item == 1

        # Fill the I/O buffer
        while True:
            try:
                queue._put_sock.send(b'x'*10000)
            except BlockingIOError:
                break

        # Make sure this doesn't fail
        await queue.put(2)
        item = await queue.get()
        assert item == 2

    kernel.run(main)

def test_uqueue_put_cancel(kernel):
    async def main():
        queue = UniversalQueue(maxsize=1)
        await queue.put(1)
        try:
            await timeout_after(0.1, queue.put(2))
            assert False
        except TaskTimeout:
            assert True

    kernel.run(main)













# guiserv2.py
#
# Another example of integrating Curio with the Tkinter
# event loop using UniversalQueue and threads.

import tkinter as tk
import threading
from curio import *

class EchoApp(object):
    def __init__(self):
        self.gui_ops = UniversalQueue(withfd=True)
        self.coro_ops = UniversalQueue()

        # Main Tk window
        self.root = tk.Tk()

        # Number of clients connected label
        self.clients_label = tk.Label(text='')
        self.clients_label.pack()
        self.nclients = 0
        self.incr_clients(0)
        self.client_tasks = set()

        # Number of bytes received label
        self.bytes_received = 0
        self.bytes_label = tk.Label(text='')
        self.bytes_label.pack()
        self.update_bytes()

        # Disconnect all button
        self.disconnect_button = tk.Button(text='Disconnect all',
                                           command=lambda: self.coro_ops.put(self.disconnect_all()))
        self.disconnect_button.pack()

        # Set up event handler for queued GUI updates
        self.root.createfilehandler(self.gui_ops, tk.READABLE, self.process_gui_ops)

    def incr_clients(self, delta=1):
        self.nclients += delta
        self.clients_label.configure(text='Number Clients %d' % self.nclients)

    def update_bytes(self):
        self.bytes_label.configure(text='Bytes received %d' % self.bytes_received)
        self.root.after(1000, self.update_bytes)

    def process_gui_ops(self, file, mask):
        while not self.gui_ops.empty():
            func, args = self.gui_ops.get()
            func(*args)

    async def echo_client(self, sock, address):
        await self.gui_ops.put((self.incr_clients, (1,)))
        self.client_tasks.add(await current_task())
        try:
            async with sock:
                while True:
                    data = await sock.recv(100000)
                    if not data:
                        break
                    self.bytes_received += len(data)
                    await sock.sendall(data)
        finally:
            self.client_tasks.remove(await current_task())
            await self.gui_ops.put((self.incr_clients, (-1,)))

    async def disconnect_all(self):
        for task in list(self.client_tasks):
            await task.cancel()

    async def main(self):
        serv = await spawn(tcp_server, '', 25000, self.echo_client)
        while True:
            coro = await self.coro_ops.get()
            await coro

    def run_forever(self):
        threading.Thread(target=run, args=(self.main,)).start()
        self.root.mainloop()

if __name__ == '__main__':
    app = EchoApp()
    app.run_forever()





# zmq RPC client example. Requires zmq_rpcserv.py to be runnig

import curio_zmq as zmq
from curio import sleep, spawn

from hello import fib

async def ticker():
    n = 0
    while True:
        await sleep(1)
        print('Tick:', n)
        n += 1

async def client(address):
    # Run a background task to make sure the message passing operations don't block
    await spawn(ticker, daemon=True)

    # Compute the first 40 fibonacci numbers
    ctx = zmq.Context()
    sock = ctx.socket(zmq.REQ)
    sock.connect(address)
    for n in range(1, 40):
        await sock.send_pyobj((fib, (n,), {}))
        result = await sock.recv_pyobj()
        print(n, result)

if __name__ == '__main__':
    zmq.run(client, 'tcp://localhost:9000')



"""A Curio websocket server.

pip install wsproto before running this.

"""
from curio import Queue, run, spawn, TaskGroup
from curio.socket import IPPROTO_TCP, TCP_NODELAY
from wsproto.connection import WSConnection, SERVER
from wsproto.events import (ConnectionClosed, ConnectionRequested, TextReceived,
                            BytesReceived)

DATA_TYPES = (TextReceived, BytesReceived)


async def ws_adapter(in_q, out_q, client, _):
    """A simple, queue-based Curio-Sans-IO websocket bridge."""
    client.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
    wsconn = WSConnection(SERVER)
    closed = False

    while not closed:
        wstask = await spawn(client.recv, 65535)
        outqtask = await spawn(out_q.get)

        async with TaskGroup([wstask, outqtask]) as g:
            task = await g.next_done()
            result = await task.join()
            await g.cancel_remaining()

        if task is wstask:
            wsconn.receive_bytes(result)

            for event in wsconn.events():
                cl = event.__class__
                if cl in DATA_TYPES:
                    await in_q.put(event.data)
                elif cl is ConnectionRequested:
                    # Auto accept. Maybe consult the handler?
                    wsconn.accept(event)
                elif cl is ConnectionClosed:
                    # The client has closed the connection.
                    await in_q.put(None)
                    closed = True
                else:
                    print(event)
            await client.sendall(wsconn.bytes_to_send())
        else:
            # We got something from the out queue.
            if result is None:
                # Terminate the connection.
                print("Closing the connection.")
                wsconn.close()
                closed = True
            else:
                wsconn.send_data(result)
            payload = wsconn.bytes_to_send()
            await client.sendall(payload)
    print("Bridge done.")


async def ws_echo_server(in_queue, out_queue):
    """Just echo websocket messages, reversed. Echo 3 times, then close."""
    for _ in range(3):
        msg = await in_queue.get()
        if msg is None:
            # The ws connection was closed.
            break
        await out_queue.put(msg[::-1])
    print("Handler done.")


def serve_ws(handler):
    """Start processing web socket messages using the given handler."""
    async def run_ws(client, addr):
        in_q, out_q = Queue(), Queue()
        ws_task = await spawn(ws_adapter, in_q, out_q, client, addr)
        await handler(in_q, out_q)
        await out_q.put(None)
        await ws_task.join()  # Wait until it's done.
        # Curio will close the socket for us after we drop off here.
        print("Master task done.")

    return run_ws


if __name__ == '__main__':
    from curio import tcp_server
    port = 5000
    print(f'Listening on port {port}.')
    run(tcp_server, '', port, serve_ws(ws_echo_server))



# Example: A simple Unix  echo server

from curio import run, unix_server


async def echo_handler(client, address):
    print('Connection from', address)
    while True:
        data = await client.recv(10000)
        if not data:
            break
        await client.sendall(data)
    print('Connection closed')


if __name__ == '__main__':
    import os
    try:
        os.remove('/tmp/curiounixecho')
    except:
        pass
    try:
        run(unix_server, '/tmp/curiounixecho', echo_handler)
    except KeyboardInterrupt:
        pass



# test_activation.py

from curio import Kernel, sleep, run
from curio.kernel import Activation

class _TestActivation(Activation):
    def __init__(self):
        self.events = []

    def activate(self, kernel):
        self.events.append('activate')

    def created(self, task):
        if task.name.endswith('main'):
            self.events.append('created')

    def running(self, task):
        if task.name.endswith('main'):
            self.events.append('running')

    def suspended(self, task, trap):
        if task.name.endswith('main'):
            self.events.append('suspended')

    def terminated(self, task):
        if task.name.endswith('main'):
            self.events.append('terminated')

def test_activation_base():
    async def main():
        await sleep(0.01)
        await sleep(0.01)
        await sleep(0.01)

    a = _TestActivation()
    run(main, activations=[a])
    assert a.events == ['activate', 'created', 'running', 'suspended', 'running', 'suspended',
                       'running', 'suspended', 'running', 'suspended', 'terminated']

def test_activation_crash():
    async def main():
        await sleep(0.01)
        raise ValueError("Dead")

    a = _TestActivation()
    kern = Kernel(activations=[a])
    try:
        kern.run(main)
        assert False
    except ValueError as e:
        assert a.events == ['activate', 'created', 'running', 'suspended', 'running', 'suspended', 'terminated']

    kern.run(shutdown=True)

class _TestActivationCreate(Activation):
    def __init__(self):
        self.events = set()

    def created(self, task):
        self.events.add(task.name.split('.')[-1])

def test_activation_count():
    async def main():
        await sleep(0.001)

    a = _TestActivationCreate()
    run(main, activations=[a])
    # There should be three tasks. main(), an in-kernel task, and a shutdown task
    assert a.events == { 'main', '_kernel_task', '_shutdown_tasks' }

